/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*
 *  ======== tst.h ========
 *  Purpose:
 *      Provides test functions
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *   ---------------------------------------------------------------------------
 *! Revision History
 *! ================
 *! 02-Nov-1999 kc: Updated function return types from void to VOID.
 *! 29-Oct-1999 kc: Added for WinCE.
 *! 21-Oct-1996 cr: Created.
 */

/* Test suite header file */
#include <os.h>

/* local definitions */
#define TST_SHOULDEQUAL     TRUE
#define TST_SHOULDNOTEQUAL  FALSE

/* stringize the macro argument */
#define val(l)              #l
#define addInfo(f, l, fmt)  "Error, "f" line "## val(l)": "## fmt
#define TST_makeFmt(fmt)    addInfo(__FILE__, __LINE__, fmt)

/*
 *  ======== TST_CheckVal ========
 *  Purpose:
 *      Validates the value based on entered parameters.
 *  Parameters:
 *      dwValidateValue:    The Value to check.
 *      dwBaseValue:        The Base value.
 *      fMustBe:            Should (Base == Value) or (Base != Value).
 *  Returns:
 *      True if the values pass, false if they do not.
 */
BOOL TST_CheckVal(DWORD dwValidateValue, DWORD dwBaseValue, BOOL fMustBe);


/*
 *  ======== TST_reportError ========
 *  Purpose:
 *      TST module's printf.
 */
VOID TST_reportError(char * fmt, ...);


/*
 *  ======== TST_setVerbose ========
 *  Purpose:
 *      Turn print statements on or off.
 *  Parameters:
 *  Returns:
 *  Requires:
 *  Ensures:
 */
VOID TST_setVerbose(BOOL flag);


/*
 *  ======== TST_verbose ========
 *  Purpose:
 *      Prints message to stderr if g_fVerbose flag set.
 */
VOID TST_verbose(char * fmt, ...);

