/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
 /*
 *  ======== anlysis.c ========
 *  Description: 
 *        Automated Environment for running GPP side Behaviour Tests 
 *        "anlysis -v" will run all tests verbose.
 *        "anlysis -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *  Usage:
 *        anlysis [-v] [-l "log path/filename"] 
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *        -v: verbose mode
 *        -l: use log and location to store results
 *        -x: DSP executable to be used in testing
 *        -m: individual module to test or ALL for all modules
 *            (Default = ALL)
 *        -n: list of API functions (see below) or ALL for all functions
 *            (Default = ALL)
 *        -#: number of times to run each API function test
 *
 * REVISION HISTORY:
 * =================
 *  14-April-2005 RAMESH GUPTA : Added Test Cases for IVA/IVA Power Management
 *! 23-Jul-2003 SWAMY MC:    Added test cases for Dynamic Linking
 *! 10-Jul-2002 SWAMY MC:    Created
 */

#include <testosal.h>
#include <stdio.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <IssueReclaim.h>
/* Function Declarations which will be used in this file */

INT sendMsg(int argc, char ** argv);
INT asyncPutGetData(int argc, char ** argv);
INT dslLoadImage(int argc, char ** argv);
INT getData(int argc, char ** argv);
INT getMsg(int argc, char ** argv);
INT putData(int argc, char ** argv);
INT putGetData(int argc, char ** argv);
INT putGetMsg(int argc, char ** argv);
INT Monitor(int argc, char **argv);
INT baseImgLoadTime_ANALYSIS(int argc, char ** argv);
INT dllLoadTime_ANALYSIS(int argc, char ** argv);
INT dllLoadTimeSP_ANALYSIS(int argc, char ** argv);
INT DSPDMMCOPY_main(int argc,char **argv);
INT shmAlloc(int argc,char **argv);
INT BssChk(int argc,char **argv);
INT nodeEnvPtr_TEST(int argc,char **argv);
INT zeroCopyMsg_Test(int argc, char **argv);
INT DynamicHeap(int argc,char **argv);
/* 
 *  ======== anlysis_main ======== 
 */
INT anlysis_main (INT argc, CHAR * argv[])
{

    CHAR        * tmpSuiteName = "ALL";
    CHAR        ** tmp_argv;
    INT         tmp_argc;
    fLoadDsp = FALSE;

    tmpSuiteName = argv[1];
    /* Now drop the first parameter and pass the rest of the
    parameters to corresponding Test Suite */
    tmp_argc = argc;  tmp_argc--;
    argv++;  tmp_argv = argv;

    if (!CSL_Strcmp(tmpSuiteName, "APUTGETDATA"))
    {
	    TST_verbose("APUTGETDATA\n") ;
        asyncPutGetData(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "LOADIMG"))
    {
	TST_verbose("LOADIMG\n") ;
        dslLoadImage(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "GETDATA"))
    {
	    TST_verbose("GETDATA\n") ;
        getData(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "GETMSG"))
    {
	TST_verbose("GETMSG\n") ;
        getMsg(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PUTDATA"))
    {
	TST_verbose("PUTDATA\n") ;
        putData(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PUTGETDATA"))
    {
	    TST_verbose("PUTGETDATA\n") ;
        putGetData(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MONITOR"))
    {
            TST_verbose("MONITOR\n") ;
            Monitor(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DMMCOPY"))
    {
	    TST_verbose("DMMCOPY\n") ;
        DSPDMMCOPY_main(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PUTGETMSG"))
    {
	TST_verbose("PUTGETMSG\n") ;
        putGetMsg(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "SENDMSG"))
    {
	TST_verbose("SENDMSG\n") ;
        sendMsg(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "NODEENVTEST"))
    {
	TST_verbose("NODEENVPTR\n") ;
        nodeEnvPtr_TEST(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "ZEROCOPYMSG"))
    {
	TST_verbose("ZEROCOPYMSG\n") ;
        zeroCopyMsg_Test(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNAMICHEAP"))
    {
	TST_verbose("DYNAMICHEAP\n") ;
        DynamicHeap(tmp_argc, tmp_argv);
    }
   else if(!CSL_Strcmp(tmpSuiteName,"BSSCHK"))
    {
        TST_verbose("BSSCHK\n");
        BssChk(tmp_argc,tmp_argv);

    }
   else if(!CSL_Strcmp(tmpSuiteName,"SHMTEST"))
    {
        TST_verbose("SHMTEST\n");
	shmAlloc(tmp_argc,tmp_argv);

    }
    else if (!CSL_Strcmp(tmpSuiteName, "IMGLOADTIME"))
    {
	TST_verbose("BASEIMAGELOADTIME\n") ;
        baseImgLoadTime_ANALYSIS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DLLLOADTIME"))
    {
	TST_verbose("DLLLOADTIME\n") ;
        dllLoadTime_ANALYSIS(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DLLLOADTIMESP"))
    {
	TST_verbose("DLLLOADTIMESPLIT\n") ;
        dllLoadTimeSP_ANALYSIS(tmp_argc, tmp_argv);
    }
   
    return 0;    
}


