/*  ============================================================================
*   File    IVALoadImage.c
*
*   Path    $(TESTSUITEROOT)\Analysis\IVALoadImage
*
*   Desc:
*      Loads the IVA Image and measures the time to load a Image.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG            Raghunandan NagarajaRao.
*   JULY 22, 2004  Based on Original changed for IVA  -RAMESH GUPTA G
*      Original Version.
*   ============================================================================
*/
#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <DSPProcessor_OEM.h>


#define	DEFNAMESIZE         80
#define DSPEXEC             "./iva.dof"
#define DEFAULTDSPUNIT      0           /* Default unit number for the board. */
#define DEFAULTIVAUNIT      1           /* Default unit number for IVA. */

static CHAR			        szLogName[DEFNAMESIZE];

static CONST  CHAR	        *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1"};
static BOOL			        fLoadDsp = FALSE, fOpenLog = FALSE;
static CONST  CHAR	        *aEnvp[DEFNAMESIZE] = {NULL};
static FILE			        *pHandle;
static INT                  NumIterations = 1;

typedef struct {

      DSP_HPROCESSOR      hProcessorIva;     /* Handle to IVA processor. */
      DSP_HNODE           hNodeIva;          /* Handle to IVA node. */
      INT                 numMsg;

}IVAMSG_TASK;

static DSP_STATUS InitializeIvaProcessor(IVAMSG_TASK *ivaMsgTask);


/* 
 *  ======== ivaLoadImage ========
 */
int ivaLoadImage(int argc, char ** argv)
{

   	IVAMSG_TASK             ivaMsgTask;
	DSP_STATUS              status = DSP_SOK;
    	fLoadDsp = FALSE;
    	INT                     opt = 0;
 
	ivaMsgTask.hProcessorIva = NULL;
	struct DSP_PROCESSORINFO       dspInfo;
        UINT                            numProcs;
        UINT                            index=0;
        INT                                     procId;


	optind = 0;
	  while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
			pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
			break;

          case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			/* wait for user */
			break;

          case 'n':
            /* Number of times to load the image */
            NumIterations = CSL_Atoi(optarg);
            break;

	  default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }



 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        printf("IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&ivaMsgTask.hProcessorIva);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");




    if (DSP_SUCCEEDED(status)) 
	{
        /* Load user specified DSP Executable and start the processor. */
		if(fLoadDsp)
			status = InitializeIvaProcessor(&ivaMsgTask);
    }


    if (ivaMsgTask.hProcessorIva) 
	{
        DSPProcessor_Detach(ivaMsgTask.hProcessorIva);
        ivaMsgTask.hProcessorIva = NULL;
    }

    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeIvaProcessor ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeIvaProcessor(IVAMSG_TASK *ivaMsgTask)
{

	  DSP_STATUS            status = DSP_SOK;
	  INT			iArgc = 1;
	  DWORD			dwStartTime;  
	  DWORD			dwAvgTime = 0;
	  DWORD			dwEndTime;
      	  DWORD         dwTotalElapsedTime;
          INT           i = 0;  
	  
      OsSpecificTimerStart(&dwStartTime);
      for( i = 0; i < NumIterations; i++)
      {
          status = DSPProcessor_Load(ivaMsgTask->hProcessorIva, iArgc, aArgv, aEnvp);  
	  if (DSP_SUCCEEDED(status))
		{
			TSTLOG_appendLog(pHandle,"Load successful in iteration %d\n",i);
		}
	  else
		{
			TSTLOG_appendLog(pHandle,"Load Failed in iteration %d\n",i);
				
		}
      }
      OsSpecificTimerStop(&dwEndTime);
      
      if (DSP_SUCCEEDED(status)) 
	  {
            OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwTotalElapsedTime);
            dwAvgTime = dwTotalElapsedTime / NumIterations;
            if(fOpenLog)
            {
		        TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
		        TSTLOG_appendLog(pHandle, "LOG: *********IVA LOAD IMAGE STATISTICS**********\n");
		        TSTLOG_appendLog(pHandle, "LOG: StartTime = %d\n", dwStartTime); 
		        TSTLOG_appendLog(pHandle, "LOG: EndTime = %d\n", dwEndTime); 
		        TSTLOG_appendLog(pHandle, "LOG: Average ElapsedTime = %d mSec\n", dwAvgTime); 
		        TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
		        TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
            }
	  }
	  else
	  {
   			/* Report Error */
			if(fOpenLog)
				TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load:IVA. STATUS = 0x%x\n", status);	
            /* Start Processor */
	  }


			status = DSPProcessor_Start(ivaMsgTask->hProcessorIva);
			if (DSP_FAILED(status)) 
			{	
                		if(fOpenLog)
			        TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start:IVA. status = 0x%x\n", status);
	  		}
			else
			{
			        TSTLOG_appendLog(pHandle, "DSPProcessor Start:IVA  Success. \n");
				
			}
      return (status);

}

