/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
*   File    PutData.c
*
*   Path    $(TESTSUITEROOT)\Analysis\PutData
*
*	Desc	Host initializes and sends a data buffer. Tests data pumping of the Host.
* 
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG             Vijay Pasam
*      Original Version.
*   ============================================================================
*/

#include <stdio.h> 
#include <getopt_test.h>
#include <dbapi.h>
#include <tst.h>
#include <tstlog.h>
#include <DSPProcessor_OEM.h>
#include <IssueReclaim.h>
#include <testosal.h>
#include <string.h>
#include <sys/time.h>

static NODEARGS		NodeArgs;


/* 
 *  ======== putData ========
 */
int putData(int argc, char ** argv)
{
    STRMCOPY_TASK   strmcopyTask;
    DSP_STATUS      status = DSP_SOK;
    fLoadDsp = FALSE;    
    

    /* Process command line arguments, open data files: */
    status = ProcessArgs(argc, argv);
    strmcopyTask.hProcessor = NULL;
    strmcopyTask.hNode = NULL;
    strmcopyTask.hOutStream = NULL;
    strmcopyTask.hInStream = NULL;

    strmcopyTask.ppInBufs = NULL;
    strmcopyTask.ppOutBufs = NULL;

    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL,&strmcopyTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (DSP_SUCCEEDED(status)) {
        /* Perform processor level initialization. */
		if(fLoadDsp)
		{
			status = InitializeProcessor(&strmcopyTask);
    }
   }

    if (DSP_SUCCEEDED(status)) {
        
        /* Perform node level initialization. */
        status = InitializeNode(&strmcopyTask);
        if (DSP_SUCCEEDED(status)) {
            
            /* Perform stream level initialization. */
            status = InitializeStreams(&strmcopyTask);
            if (DSP_SUCCEEDED(status)) {
                
                /* Run task. */
                status = RunTask(&strmcopyTask);
                if (DSP_FAILED(status)) 
				{
					if(fOpenLog)
						TSTLOG_appendLog(pHandle, "RunTask failed.\n");
                }
            }
        }
    }
    
    /* Close opened files. */
    CleanupStreams(&strmcopyTask);
    CleanupNode(&strmcopyTask);
    CleanupProcessor(&strmcopyTask);
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProcessor ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
	INT					iArgc = 1;

	if (DSP_SUCCEEDED(status)) 
	{
		status = DSPProcessor_Load(copyTask->hProcessor, iArgc, (CONST CHAR **)aArgv, (CONST CHAR **)aEnvp);  
		if (DSP_SUCCEEDED(status)) 
		{
				/* Start Processor */
				status = DSPProcessor_Start(copyTask->hProcessor);
				if (DSP_FAILED(status)) 
				{
					if(fOpenLog)
						TSTLOG_appendLog(pHandle, "DSPProcessor Start FAIL.STATUS = %x\n", status);
				}
		}
		else
		{
			if(fOpenLog)	
				TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. STATUS = %x\n", status);
		}
	}
	return (status);
}


/*
 *  ======== InitializeNode ========
 *  Perform node related initialization.
 */
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_STRMATTR    attrs;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    uuid = inoutuuid;

    attrs.uBufsize = NodeArgs.iBufferSize;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.uAlignment = 0;
    attrs.uTimeout = DEFAULTTIMEOUT;
	switch(NodeArgs.iMode)
	{
		case STRMMODE_PROCCOPY : 
			attrs.lMode = STRMMODE_PROCCOPY; 
			copyTask->nStrmMode = STRMMODE_PROCCOPY ;
			break;
		case STRMMODE_ZEROCOPY : 
			attrs.lMode = STRMMODE_ZEROCOPY; 
			attrs.uSegid = DSP_SHMSEG0;
			 copyTask->nStrmMode = STRMMODE_ZEROCOPY ;
			break;
		case STRMMODE_RDMA : 
			attrs.lMode = STRMMODE_RDMA; 
			 copyTask->nStrmMode = STRMMODE_RDMA ;
			break;
		default:
			attrs.lMode = STRMMODE_PROCCOPY; 
			break;
	}
    
    if (copyTask->nStrmMode == STRMMODE_ZEROCOPY)
    {
            attrs.uSegid = DSP_SHMSEG0;  /* Use SM for zero-copy */
    }
    else 
    {
            attrs.uSegid = DEFAULTSEGMENT;
    }
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    nodeAttrIn.uTimeout = DEFAULTTIMEOUT;
    nodeAttrIn.iPriority = 5;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif


    /* Allocate the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,
                &nodeAttrIn, &copyTask->hNode);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Connect the strmcopy node to the GPP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, copyTask->hNode,
                0, &attrs);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Connect(copyTask->hNode, 0, (DSP_HNODE)DSP_HGPPNODE,
            0, &attrs);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Connect failed. Status = 0x%x\n",
                    status);
        }
    }

    /* Create the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Create(copyTask->hNode);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Create failed. Status = 0x%x\n",
                status);
        }
    }
    
    /* Start the strmcopy node on the DSP. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPNode_Run(copyTask->hNode);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",
                status);
        }
    }
    
    return (status);
}


/*
 *  ======== InitializeStreams ========
 *  Perform stream related initialization.
 */
static DSP_STATUS InitializeStreams(STRMCOPY_TASK  *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    

    struct DSP_STREAMATTRIN    attrs;

    attrs.cbStruct = sizeof(struct DSP_STREAMATTRIN);
    attrs.uTimeout = DEFAULTTIMEOUT;
    attrs.uAlignment = 0;
    attrs.uNumBufs = DEFAULTNBUFS;
    attrs.lMode = copyTask->nStrmMode;

    if (copyTask->nStrmMode == STRMMODE_PROCCOPY)
    {
    	attrs.uSegment = DEFAULTSEGMENT;
    }
    else
    {
	    attrs.uSegment = DSP_SHMSEG0;    /* Use shared memory segment 0 */
    }

    /* Open an input data stream from the host to the strmcopy node. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPStream_Open(copyTask->hNode, DSP_TONODE, 0, 
            &attrs, &copyTask->hOutStream);
        if (DSP_FAILED(status))
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_Open of input stream failed. Status = 0x%x\n",
                status);
        }
    }

    /* Open an output stream from the strmcopy node to the host. */
    if (DSP_SUCCEEDED(status)) {
        status = DSPStream_Open(copyTask->hNode, DSP_FROMNODE, 0, 
            &attrs, &copyTask->hInStream);
        if (DSP_FAILED(status))
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_Open of output stream failed. Status = 0x%x\n",
                status);
        }
    }
    
	/* Allocate and issue buffer to input data stream. */
    if (DSP_SUCCEEDED(status)) {
        copyTask->ppOutBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hOutStream, 
            NodeArgs.iBufferSize, copyTask->ppOutBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_AllocateBuffers for input stream failed. "
                "Status = 0x%x\n", status);
        }
    }
    
    /* Allocate and issue buffer to output data stream. */
	if (DSP_SUCCEEDED(status)) {
        copyTask->ppInBufs = (BYTE**)OsSpecificMemAlloc(sizeof(BYTE*) * DEFAULTNBUFS);

        status = DSPStream_AllocateBuffers(copyTask->hInStream, 
            NodeArgs.iBufferSize, copyTask->ppInBufs, DEFAULTNBUFS);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, 
                "DSPStream_AllocateBuffers for output stream failed. "
                "Status = 0x%x\n", status);
        }
    }

    return (status);
}


/*
 *  ======== RunTask ========
 *  Run strmcopy task.
 */
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask)
{
		DSP_STATUS      status = DSP_SOK;
		BYTE            *pInBuf = NULL;
		BYTE            *pOutBuf = NULL;
		DWORD           dwArg = 0;
		DWORD           dwBufSize = NodeArgs.iBufferSize;
		UINT			k;
		DWORD			dwStartTime;  
		DWORD			dwElapsedTime;
		DWORD			dwEndTime;
		ULONG			ulNumBytesTransferred = 0;
		ULONG			ulBytesPerSec = 0;
		DWORD			dwStartTimeTmp;  
		DWORD			dwElapsedTimeTmp;
		DWORD			dwEndTimeTmp;
		struct 			timeval *tvBeg;
		struct 			timeval	*tvEnd;
		struct 			timeval	*tvElapsed;
		struct 			timeval	*tvMainBeg;
		struct 			timeval	*tvMainEnd;
		struct 			timezone *tz;
		INT 			TimerResult=0;
		
		tvBeg =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
		tvEnd =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
		tvElapsed =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
		tvMainBeg =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
		tvMainEnd =  (struct timeval*)OsSpecificMemAlloc(sizeof(struct timeval));
		tz =  (struct timezone*)OsSpecificMemAlloc(sizeof(struct timezone));

		pInBuf = copyTask->ppInBufs[0];
        	pOutBuf = copyTask->ppOutBufs[0];

		status = SendTestConfiguration((SHORT **)&pInBuf, (SHORT **)&pOutBuf, copyTask, 0, dwBufSize);

		if(DSP_SUCCEEDED(status))
		{
			TimerResult = gettimeofday(tvMainBeg,tz);	
			if( TimerResult != 0)
			{
				TSTLOG_appendLog(pHandle, "GetTimeOfDay failed\n");
			}
			OsSpecificTimerStart(&dwStartTime);

            for( k = 0; k < NodeArgs.iNumTxfrs; k++)
			{
					status = DSPStream_Issue(copyTask->hOutStream, pOutBuf,	NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);
					if (DSP_FAILED(status)) 
					{
						if(fOpenLog)
							TSTLOG_appendLog(pHandle, "DSPStream_Issue %d failed.status = 0x%x\n", k,status);
                        				break;
					}

					status = DSPStream_Reclaim(copyTask->hOutStream, &pOutBuf, &dwBufSize, NULL, &dwArg);

                    if (DSP_FAILED(status)) 
					{
                        if(fOpenLog)
							TSTLOG_appendLog(pHandle, "DSPStream_Reclaim %d failed.status = 0x%x\n", k,status);
                        				break;
					}

#if 0
                    if ((k % 500) == 0)
                    {
                        TST_verbose("Issue Reclaim: %d\n",k) ;
                    }
#endif
			}        

			OsSpecificTimerStop(&dwEndTime);

			TimerResult = gettimeofday(tvMainEnd,tz);	
			if( TimerResult != 0)
			{
				TSTLOG_appendLog(pHandle, "GetTimeOfDay failed\n");
			}
			if (tvMainBeg->tv_usec > tvMainEnd->tv_usec) { 
  				 tvMainEnd->tv_usec += 1000000; 
   				 tvMainEnd->tv_sec--; 
			}
		
			OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);
	
			if(DSP_SUCCEEDED(status))
            		{	
			    ulNumBytesTransferred = dwBufSize * NodeArgs.iNumTxfrs;
			    ulBytesPerSec = (ulNumBytesTransferred / dwElapsedTime) * 1000;
			    if(fOpenLog)
			    {
				    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
				    TSTLOG_appendLog(pHandle, "LOG: *********PUT DATA STATISTICS**********\n");	
                    		    TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = PASS\n");
				    TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
				    TSTLOG_appendLog(pHandle, "LOG: StartTime = %u\n", dwStartTime); 
				    TSTLOG_appendLog(pHandle, "LOG: EndTime = %u\n", dwEndTime); 
				    TSTLOG_appendLog(pHandle, "LOG: ElapsedTime = %u sec and %u uSec\n", (tvMainEnd->tv_sec - tvMainBeg->tv_sec), (tvMainEnd->tv_usec - tvMainBeg->tv_usec)); 
				    TSTLOG_appendLog(pHandle, "LOG: Throughput = %d Bytes/sec\n", ulBytesPerSec); 
				    TSTLOG_appendLog(pHandle, "LOG: ulNumBytesTransferred = %d\n", ulNumBytesTransferred);
				    TSTLOG_appendLog(pHandle, "LOG: **********END STATISTICS***********\n");
				    TSTLOG_appendLog(pHandle, "LOG: ***********************************\n");
			    }
            		}
            		else
            		{
                		if(fOpenLog)
                		{
                    			TSTLOG_appendLog(pHandle, "LOG: BufferSize = %d\n", dwBufSize);
                    			TSTLOG_appendLog(pHandle, "LOG: FINAL STATUS = 0x%x\n", status);
                		}
                	TST_verbose("status = 0x%x \n",status); 
            		}
		}
			return (status);
}

/*
 *  ======== CleanupStreams ========
 *  Perform stream related cleanup.
 */ 
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask)
{
    struct DSP_STREAMINFO  streamInfo;
    DSP_STATUS      status = DSP_SOK;
    DWORD           dwArg = 0;
    DWORD           dwBufsize = NodeArgs.iBufferSize;
    /* Reclaim and free stream buffers. */
    if (copyTask->hOutStream) 
	{
        DSPStream_Idle(copyTask->hOutStream, TRUE);
        status = DSPStream_GetInfo(copyTask->hOutStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppOutBufs) 
		{
            /* Reclaim the buffer if it's still in the stream */
            if (streamInfo.uNumberBufsInStream) 
			{
                status = DSPStream_Reclaim(copyTask->hOutStream,
                    &copyTask->ppOutBufs[0], &dwBufsize, NULL, &dwArg);
                if (DSP_FAILED(status)) 
				{
                    if(fOpenLog)
					    TSTLOG_appendLog(pHandle,  "DSPStream_Reclaim of Output buffer failed. "
                        "Status = 0x%x\n", status);
				}
            }
            status = DSPStream_FreeBuffers(copyTask->hOutStream, 
                    copyTask->ppOutBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) 
			{
                if(fOpenLog) 
					TSTLOG_appendLog(pHandle,  "DSPStream_FreeBuffer of input buffer failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->ppOutBufs = NULL;
            /* Close open streams. */
            status = DSPStream_Close(copyTask->hOutStream);
            if (DSP_FAILED(status)) 
			{
                if(fOpenLog)
					TSTLOG_appendLog(pHandle, "DSPStream_Close of Output stream failed. "
                    "Status = 0x%x\n", status);
            }

            copyTask->hOutStream = NULL;
        }
    }
    
    if (copyTask->hInStream) 
    {
        DSPStream_Idle(copyTask->hInStream, TRUE);

        /* Need to determine the number of buffers left in the stream */
        status = DSPStream_GetInfo(copyTask->hInStream, &streamInfo,
            sizeof(streamInfo));

        if (copyTask->ppInBufs) 
        {
            if (streamInfo.uNumberBufsInStream) 
            {
                status = DSPStream_Reclaim(copyTask->hInStream, 
                        &copyTask->ppInBufs[0], &dwBufsize, NULL, &dwArg);
                if (DSP_FAILED(status)) 
                {
				    if(fOpenLog)
					    TSTLOG_appendLog(pHandle, 
                        "DSPStream_Reclaim of Input buffer failed. "
                        "Status = 0x%x\n", status);
            	}
            }

            /* Free buffer. */
            status = DSPStream_FreeBuffers(copyTask->hInStream, 
                copyTask->ppInBufs, DEFAULTNBUFS);
            if (DSP_FAILED(status)) 
            {
				if(fOpenLog)
					TSTLOG_appendLog(pHandle, 
                    "DSPStream_FreeBuffer of output buffer failed. "
                    "Status = 0x%x\n", status);
            }
            copyTask->ppInBufs = NULL;
        }

        status = DSPStream_Close(copyTask->hInStream);
    
        if (DSP_FAILED(status)) 
	    {
            TSTLOG_appendLog(pHandle, "DSPStream_Close of output stream failed . "
            "Status = 0x%x\n", status);
        }

        copyTask->hInStream = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupNode ========
 *  Perform node related cleanup.
 */ 
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;
    DSP_STATUS      exitStatus;
    
    if (copyTask->hNode) {

        // Terminate DSP node.
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Terminate failed: 0x%x\n", status);
        }

        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPNode_Delete failed: 0x%x\n", status);
        }

        copyTask->hNode = NULL;
    }

    return (status);
}


/*
 *  ======== CleanupProcessor ========
 *  Perform processor related cleanup.
 */ 
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask)
{
    DSP_STATUS      status = DSP_SOK;

    if (copyTask->hProcessor) {
        /* Detach from processor. */
        status = DSPProcessor_Detach(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
		{
			if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPProcessor_Detach failed.\n");
        }

        copyTask->hProcessor = NULL;
    }

    return (status);
}


/* 
 *  ======== ProcessArgs ========
 *  Process command line arguments for this sample, returning input and
 *  output file handles.
 */
static DSP_STATUS ProcessArgs(int argc, char ** argv)
{
    DSP_STATUS      status = DSP_SOK;
	INT             opt = 0;
	BOOL			bUseLog=FALSE;
    NodeArgs.iSegmentId = DEFAULTSEGMENT;
	NodeArgs.iMode  = 0;
	optind = 0;
        while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:b:n:ZPR:")) != EOF) {
        switch (opt) {
         case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
			break;

	      case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
			bUseLog = TRUE;
            pHandle = TSTLOG_openLog(szLogName);
			fOpenLog = TRUE;
	        break;

          case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
			fLoadDsp = TRUE;
			break;

		case 'b':
            /* set Buffer Size */
            NodeArgs.iBufferSize = CSL_Atoi(optarg);
			break;

		case 'n':
            /* set Number of transfers */
            NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
			break;

		case 'Z':
			/* set Transfer Mode as STRMMODE_ZEROCOPY */
			NodeArgs.iMode = STRMMODE_ZEROCOPY;
			NodeArgs.iSegmentId = DSP_SHMSEG0;
			break;
		case 'R':
			/* set Transfer Mode as STRMMODE_RDMA */
			NodeArgs.iMode = STRMMODE_RDMA;
			break;
		
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }
	
	    return (status);
}


static DSP_STATUS SendTestConfiguration(SHORT **pInBuf, 
										SHORT **pOutBuf, 
										STRMCOPY_TASK *copyTask, 
										DWORD dwSendBufSize, 
										DWORD dwRecvBufSize)
{

		DSP_STATUS status = DSP_SOK;
		UINT			k;
		DWORD			dwArg;
		(*pOutBuf)[0] = dwSendBufSize / 2;
		(*pOutBuf)[1] = dwRecvBufSize / 2;
		(*pOutBuf)[2] = NodeArgs.iNumTxfrs & 0xffff ;
		(*pOutBuf)[3] = NodeArgs.iNumTxfrs >> 16 ;

		for ( k = 4; k < NodeArgs.iBufferSize / 2; k++)
		{
				(*pOutBuf)[k] = k;
		}

		status = DSPStream_Issue(copyTask->hOutStream, (BYTE *)*pOutBuf, 
        NodeArgs.iBufferSize, NodeArgs.iBufferSize, 0);
        if (DSP_FAILED(status)) 
		{
            if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPStream_Issue %d failed.\n", k);
        }

		status = DSPStream_Reclaim(copyTask->hOutStream, 
						(BYTE **)pOutBuf, &dwRecvBufSize, NULL, &dwArg);
        if (DSP_FAILED(status)) 
		{
            if(fOpenLog)
				TSTLOG_appendLog(pHandle, "DSPStream_Reclaim %d failed.\n", k);
        }
		return status;
}
