/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DoubleImageLoad.h
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\Include
 * 
 *   Desc:   Header file for Double Image Load Test Case
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 27, 2002    REF=ORG             Raghunandhan NagarajaRao
 *      Original Version.
 *   ============================================================================
 */


#include <dbapi.h>

/* Hardcoded sample parameters: */
#define DEFAULTBUFSIZE              1024        /* Default buffer size in bytes. */
#define DEFAULTDSPUNIT                 0        /* Default unit number for the board. */

#define DEFAULTNBUFS                   1        /* Default # of buffers. */
#define STRMCOPY_TASK_ID TEXT("strmcopy")
#define ARGSIZE                       32
#define DSPWORDSIZE                    2           /* 1 DSP word = 2 bytes. */

#define	DEFNAMESIZE                   80
#define DEFAULTSEGMENT                 0

#define DSPEXECFIRSTIMAGE             "dataxfr_nortdxidl.dof"
#define DSPEXECSECONDIMAGE            "inout_nortdxidl.dof"

static CHAR			                  szLogNameDdsp[DEFNAMESIZE];
static CHAR			                  *aFirstImage[DEFNAMESIZE] = { DSPEXECFIRSTIMAGE, "arg1"};
static CHAR			                  *aSecondImage[DEFNAMESIZE] = { DSPEXECSECONDIMAGE, "arg1"};
static BOOL			                  fLoadFirstDsp = FALSE;
static BOOL			                  fLoadSecondDsp = FALSE;
static BOOL			                  fOpenLog = FALSE;
static CHAR			                  *aEnvp[DEFNAMESIZE] = {NULL};
static FILE			                  *pHandleDdsp;

/* Stream copy task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    struct DSP_UUID            hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE           hNode;          /* Handle to node. */
    DSP_HSTREAM         hOutStream;     /* Handle to node input stream. */
    DSP_HSTREAM         hInStream;      /* Handle to node output stream. */
    BYTE                **ppInBufs;     /* Input stream buffers. */
    BYTE                **ppOutBufs;    /* Output stream buffers. */
    struct DSP_NOTIFICATION    hNotification;  /* DSP notification object. */
} STRMCOPY_TASK; 


typedef struct {
	UINT	iBufferSize;
	UINT	iSegmentId;
	UINT	iMode;
	UINT	iNumTxfrs;
	BOOL	bCleanupFlag;
}NODEARGS;


/* Forward declarations: */
static DSP_STATUS ProcessArgs(int argc, char ** argv);

/* Initialization and cleanup routines. */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask);
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask);
static DSP_STATUS InitializeStreams(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask);
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask);

/* UUID's for PutGetData, PutData and GetData */
/* strmcopy node */

/* {F95CA076_EE7D_11D5_A8DD_00B0D055F6D1 } */
static const struct DSP_UUID inoutuuid = {	
	0xF95CA076, 0xEE7D, 0x11d5, 0xa8, 0xdd, {
		0x0, 0xb0, 0xd0, 0x55, 0xf6, 0xd1}
};


/* UUId for Dataxfr */
/* {F95CA072_EE7D_11D5_A8DD_00B0D055F6D1}*/
static const struct DSP_UUID dataxfruuid = {
	0xf95ca072, 0xee7d, 0x11d5, 0xa8, 0xdd, {
	0x00, 0xb0, 0xd0, 0x55, 0xf6, 0xd1    }
};
