/*
*  Copyright 2004 by Texas Instruments Incorporated.
*  All rights reserved. Property of Texas Instruments Incorporated.
*  Restricted rights to use, duplicate or disclose this code are
*  granted through contract.
*
*  @(#) DSP/BIOS Bridge 
*/
/*  ============================================================================
*   File    SendMsg.c
*
*   Path    $(TESTSUITEROOT)\Analysis\SendMsg
* 
*   Desc    Tests Message pumping of DSP.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2003
*
*   ============================================================================
*   Revision History
*
*   SEP 15, 2002  REF=ORG             Raghunandan NagarajaRao
*      Original Version.
*   ============================================================================
*/

#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <msgtxfr.h>


#define MMU_FAULT_REQUEST       (RMS_USER + 17)


/* 
*  ======== sendMsg ========
*/
int mmuFaultTestMsg(int argc, char ** argv)
{
    
    COPY_TASK       copyTask;
    NODEARGS        NodeArgs;
    DSP_STATUS      status = DSP_SOK;
    INT             opt = 0;
    
    fLoadDsp = FALSE;
    copyTask.hProcessor = NULL;
    copyTask.hNode = NULL;
    copyTask.hInStream = NULL;
    
    optind = 0;
    while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:s:ZPR:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            break;
            
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            pHandle = TSTLOG_openLog(szLogName);
            fOpenLog = TRUE;
            TSTLOG_appendLog(pHandle, "Zero Copy Data Transfer:\n");
            TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
            break;
            
        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            fLoadDsp = TRUE;
            TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
            break;
            
        case 'n':
            /* set Number of transfers */
            NodeArgs.iNumTxfrs = CSL_Atoi(optarg);
            if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Number of Data Transfers =  %i\n", NodeArgs.iNumTxfrs);
            break;
            
        case 'Z':
            /* set Transfer Mode as STRMMODE_ZEROCOPY */
            NodeArgs.iMode = STRMMODE_ZEROCOPY;
            NodeArgs.iSegmentId = DSP_SHMSEG0;
            if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Transfer Mode Set to =  %i\n", STRMMODE_ZEROCOPY);
            break;
        case 'R':
            /* set Transfer Mode as STRMMODE_RDMA */
            NodeArgs.iMode = STRMMODE_RDMA;
            if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Transfer Mode Set to =  %i\n", STRMMODE_RDMA);
            break;
            
        default:
            TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            break;
        }
    }
    
    /* Attach to processor */
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }
    
    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &copyTask.hProcessor);
        if(DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach Pass.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
    if (DSP_SUCCEEDED(status)) 
    {
        /* Load user specified DSP Executable and start the processor. */
        if(fLoadDsp)
            status = InitializeProcessor(&copyTask);
    }
    
    if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
        status = InitializeNode(&copyTask, &NodeArgs);
        if (DSP_SUCCEEDED(status)) 
        {
            status = RunTask(&copyTask, &NodeArgs);
            if (DSP_SUCCEEDED(status)) 
            {
                if(fOpenLog)
                    TSTLOG_appendLog(pHandle,  "RunTask Succedded\n");
            }
        }
    }
    
    CleanupNode(&copyTask);
    if (copyTask.hProcessor) 
    {
        DSPProcessor_Detach(copyTask.hProcessor);
        copyTask.hProcessor = NULL;
    }
    return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
*  ======== InitializeProcessor ========
*  Perform processor related initialization.
*/
static DSP_STATUS InitializeProcessor(COPY_TASK *copyTask)
{
    DSP_STATUS          status = DSP_SOK;
    INT                 iArgc = 1 ;
    
    status = DSPProcessor_Load(copyTask->hProcessor, iArgc, aArgv, aEnvp);  
    if (DSP_SUCCEEDED(status)) 
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded\n");
        status = DSPProcessor_Start(copyTask->hProcessor);
        if (DSP_FAILED(status)) 
        {
            if(fOpenLog)
                TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start.Status = 0x%x\n", status);
        }
    }
    else
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status = 0x%x\n",status);
    }
    
    return (status);
}

/*
*  ======== InitializeNode ========
*  Perform node related initialization.
*/
static DSP_STATUS InitializeNode(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    BYTE            argsBuf[ARGSIZE + sizeof(ULONG)];
    struct DSP_CBDATA     *pArgs;
    struct DSP_NODEATTRIN  nodeAttrIn;
    struct DSP_UUID        uuid;
    DSP_STATUS      status = DSP_SOK;
    
    uuid = Msguuid;
    
    nodeAttrIn.uTimeout = 1000;
    nodeAttrIn.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrIn.uProfileID = 0xff;
#endif
    
    pArgs = (struct DSP_CBDATA *)argsBuf;
    pArgs->cbData = ARGSIZE;
    pArgs->cData[0] = 0;
    
    /* Allocate the message node. */
    status = DSPNode_Allocate(copyTask->hProcessor, &uuid, pArgs,&nodeAttrIn, &copyTask->hNode);
    if (DSP_SUCCEEDED(status)) 
    {
        /* Create the msg node on the DSP. */
        status = DSPNode_Create(copyTask->hNode);
        
        if (DSP_SUCCEEDED(status)) 
        {
            /* Start the msg node on the DSP. */
            status = DSPNode_Run(copyTask->hNode);
            if (DSP_FAILED(status)) 
            {
                if(fOpenLog)
                    TSTLOG_appendLog(pHandle, "DSPNode_Run failed. Status = 0x%x\n",status);
            }
        }
        else 
        {
            if(fOpenLog)
                TSTLOG_appendLog(pHandle,"DSPNode_Create failed. Status = 0x%x\n",status);
        }
    }
    else 
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle, "DSPNode_Allocate failed. Status = 0x%x\n",status);
    }
    return (status);
}

/*
*  ======== RunTask ========
*  Run message task.
*/
static DSP_STATUS RunTask(COPY_TASK *copyTask, NODEARGS *NodeArgs)
{
    DSP_STATUS      status = DSP_SOK;
    UINT            uTimeout = 10000;
    struct DSP_MSG         msg;
    UINT            k;
    DWORD           dwStartTime;  
    DWORD           dwElapsedTime;
    DWORD           dwEndTime;
    ULONG           ulNumBytesTransferred = NodeArgs->iNumTxfrs;
    ULONG           ulBytesPerSec;
    
    struct DSP_NOTIFICATION*   ahEvents[2];
    struct DSP_PROCESSORSTATE  procState;
    INT         index=0;
    
    OsSpecificTimerStart(&dwStartTime);
    
    copyTask->notifications = (struct DSP_NOTIFICATION* *)OsSpecificMemAlloc(2 * sizeof(struct DSP_NOTIFICATION*));
    copyTask->nodeEventMask = DSP_MMUFAULT;
    TSTLOG_appendLog(pHandle,"Event Mask = 0x%x\n", copyTask->nodeEventMask);
    copyTask->notifications[0] = OsSpecificMemAlloc(sizeof(struct DSP_NOTIFICATION));
    copyTask->notifications[1] = 0;
    
    /* Register for Notifs for Proc State */
    status = DSPProcessor_RegisterNotify(copyTask->hProcessor, DSP_MMUFAULT, DSP_SIGNALEVENT, &copyTask->notifications[0]);
    if (DSP_FAILED(status))
    {
        TSTLOG_appendLog(pHandle,"DSPProcessor_RegisterNotify failed status = 0x%x \n",status);
    }
    else
    {
        TSTLOG_appendLog(pHandle,"DSPProcessor_RegisterNotify passed\n");
    }
    
    
    /* Set msg SM buffer descriptor cmd code. */
    msg.dwCmd =  MMU_FAULT_REQUEST;
    msg.dwArg1 = k;
    status = DSPNode_PutMessage(copyTask->hNode, &msg, uTimeout);
    
    if (DSP_FAILED(status)) 
    {
        if(fOpenLog)
            TSTLOG_appendLog(pHandle, "DSPNode_PutMessage failed. Status = 0x%x\n", status);
        return (status);
    }
    
    ahEvents[0] = &copyTask->notifications[0];
    fprintf(stdout, "calling DSPManager_WaitForEvents ...\n");
    status = DSPManager_WaitForEvents(ahEvents, 1, &index, 10000);
    
    /* Process events: */
    if (DSP_SUCCEEDED(status) && index == 0) 
    {
        fprintf(stdout, "Received MMU Fault From DSP.\n");
        
        DSPProcessor_GetState(copyTask->hProcessor, &procState, sizeof(procState));
        
        fprintf(stdout, " dwErrMask = %0x \n",procState.errInfo.dwErrMask);
        fprintf(stdout, " dwVal1 = %0x \n",procState.errInfo.dwVal1);
        fprintf(stdout, " dwVal2 = %0x \n",procState.errInfo.dwVal2);
        fprintf(stdout, " dwVal3 = %0x \n",procState.errInfo.dwVal3);
        fprintf(stdout, "MMU Fault Test Passed.\n");
    }
    else {
        fprintf(stdout, "Error in WaitForEvents- status is 0x%x\n",status);
        fprintf(stdout, "MMU Fault Test Failed.\n");
    }
    OsSpecificTimerStop(&dwEndTime);
    
    OsSpecificTimerDifference(&dwEndTime, &dwStartTime, &dwElapsedTime);
    
    return (status);
}

/*
*  ======== CleanupNode ========
*  Perform node related cleanup.
*/ 
static DSP_STATUS CleanupNode(COPY_TASK *copyTask)
{
    DSP_STATUS      status  = DSP_SOK;
    DSP_STATUS      exitStatus = DSP_SOK;
    if (copyTask->hNode) {
        
        /* Terminate the node */
        status = DSPNode_Terminate(copyTask->hNode, &exitStatus);
        if (DSP_FAILED(status)) 
        {
            //if(fOpenLog)
                //TSTLOG_appendLog(pHandle,  "DSPNode_Terminate failed.status = 0x%x\n", status);
        }
        /* Delete DSP node. */
        status = DSPNode_Delete(copyTask->hNode);
        if (DSP_FAILED(status)) 
        {
            //if(fOpenLog)
                //TSTLOG_appendLog(pHandle, "DSPNode_Delete failed.\n");
        }
        
        copyTask->hNode = NULL;
    }
    
    return (status);
}
