/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge
 */
/*  ============================================================================
*   File    DvfsTest.c
*
*   Path    $(TESTSUITEROOT)\Behaviour\PwrMgmt
*
*   Desc    Tests Power Mangement features.
*
*   Rev     1.0.0
*
*   This computer program is copyright to Texas Instruments Incorporated.
*   The program may not be used without the written permission
*   of Texas Instruments Incorporated. or against the terms and conditions
*   stipulated in the agreement under which this program has been
*   supplied.
*
*   (c) Texas Instruments Incorporated 2008
*
*   ============================================================================
*   Revision History
*
*   MAR 29, 2008  REF=ORG             Ramesh Gupta Guntha
*      Original Version.
*   ============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h> 
#include <dbapi.h>
#include <rms_sh.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <string.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>   
#include <msgtxfr.h>


#define DEFAULTDELAY  "50"	/* default ping delay                    */
#define DEFAULTMSGS   50	/* default message count                 */
#define DSPUNIT       0		/* Default (first) DSP ID                */
#define ARGSIZE       32	/* Size of arguments to Pwr  node.       */
#define MAXNAMELEN    64	/* Max length of event name.             */
#define MAXMSGLEN     128	/* Max length of MessageBox msg.         */

#define PING           0x01
#define MIN_OPP 1
#define MAX_OPP 4
/* PWR_TI_UUID = 4DB7324B_43E2_45C4_AD54_EAEE0C4FA9FE */
static const struct DSP_UUID PWR_TI_uuid = {
	0x4db7324b, 0x43e2,0x45c4,0xad,0x54,{0xea, 0xee, 0x0c, 0x4f, 0xa9, 0xfe} };

/* Pwr task context data structure - passed between functions in this module */
struct PWR_TASK {
	DSP_HPROCESSOR hProcessor;	/* Handle to processor.     */
	DSP_HNODE hNode;	/* Handle to node.                  */
	HANDLE hEvent;		/* Event to be signalled by DSP     */
	UINT msgCount;		/* Number of messages DSP sends     */
	UINT Cmd;		/* TestCase Number		    */
	UINT minOpp;		/* Value of Minimum Opp supported   */
	UINT maxOpp;		/* Value of Maximum Opp supported   */
	UINT Load;		/* Value of DSP Load                */
} ;

/* Argument structure for DSPNode_Allocate():                           */
struct PWR_NODEDATA {
	ULONG cbData;
	BYTE cData[ARGSIZE];
} ;

/* Forward Declarations */
static DSP_STATUS ProcessArgs(int argc, char **argv, UINT *pMsgCnt,
					struct PWR_NODEDATA *argsBuf);
static DSP_STATUS InitializeProc(struct PWR_TASK *pwrTask);
static DSP_STATUS CreateNode(struct PWR_TASK * pwrTask,
			struct PWR_NODEDATA * argsBuf);
static DSP_STATUS RunNode(struct PWR_TASK * pwrTask);
static DSP_STATUS DestroyNode(struct PWR_TASK * pwrTask);
static DSP_STATUS DetachProcessor(struct PWR_TASK * pwrTask);

/*
 *  ======== main ======= */
int DvfsTest_main(int argc, char **argv)
{
	struct PWR_TASK pwrTask;	/* Pwr task context                    */
	UINT msgCount;		/* Number of messages DSP sends         */
	struct PWR_NODEDATA argsBuf;	/* Task Node args.                      */
	DSP_STATUS status = DSP_SOK;
 	struct DSP_PROCESSORINFO       dspInfo;
	UINT	numProcs;
	UINT	index=0;
	INT	procId=0;
	INT	opt=0;

	optind = 0;
	pwrTask.hProcessor = NULL;
	pwrTask.hNode = NULL;
	pwrTask.hEvent = NULL;
	pwrTask.minOpp = MIN_OPP;
	pwrTask.maxOpp = MAX_OPP;
	pwrTask.Load = NULL;
	pwrTask.Cmd = 4; /*Bydefault running TestCase 4*/
	pwrTask.msgCount = msgCount;

	while ((opt = getopt_test(argc, (char * const)argv, "+vl:x:n:L:m:M:t:PR:")) != EOF) {
 	switch (opt) {
		case 'v':
        	/* verbose */
		TST_setVerbose(TRUE);
        	break;

	      	case 'l':
            	/* use log and store with name location */
            	strncpy(szLogName, optarg, sizeof(szLogName));
		pHandle = TSTLOG_openLog(szLogName);
		fOpenLog = TRUE;
            	TSTLOG_appendLog(pHandle, "Log File Set to %s\n:", szLogName);
		break;

          	case 'x':
           	/* use given executable */
	        aArgv[0] = (char *) optarg;
		fLoadDsp = TRUE;
		TSTLOG_appendLog(pHandle, "New Executable %s\n:", aArgv[0]);
		break;

		case 'n':
            	/* set Number of transfers */
            	pwrTask.msgCount = CSL_Atoi(optarg);
		if(fOpenLog)
			TSTLOG_appendLog(pHandle, "Number of Iterations =  %i\n",pwrTask.msgCount);
		break;

		case 't':
            	/* set command value*/
            	pwrTask.Cmd = CSL_Atoi(optarg);
		if(fOpenLog)
			TSTLOG_appendLog(pHandle,"Executing Test Case#%d\n", pwrTask.Cmd);
		break;
		
		case 'm':
		/* set minimum Opp Value*/
            	pwrTask.minOpp = CSL_Atoi(optarg);
		if(fOpenLog)
			TSTLOG_appendLog(pHandle,"Setting Minimum OPP value to %d\n",pwrTask.minOpp);
		break;

		case 'M':
		/* set Max Opp value */
            	pwrTask.maxOpp = CSL_Atoi(optarg);
		if(fOpenLog)
			TSTLOG_appendLog(pHandle,"Setting Max OPP value to %d\n",pwrTask.maxOpp);
		break;
		
		case 'L':
		/* set IVA Load */
            	pwrTask.Load = CSL_Atoi(optarg);
		if(fOpenLog)
			TSTLOG_appendLog(pHandle,"Setting Load to %d\n",pwrTask.Load);
		break;

        	default:
        	TSTLOG_appendLog(pHandle, "Invalid option: %c\n", opt);
            	break;
        }
      }
    /* Attach to processor */
    
    while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
        if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
            TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
            procId = index;
            status = DSP_SOK;
            break;
        }
        index++;
    }

    /* Attach to an available DSP (in this case, the 1st DSP). */
    if (DSP_SUCCEEDED(status)) {
        
        status = DSPProcessor_Attach(procId, NULL, &pwrTask.hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
    else {
        TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
    }
    
	if (DSP_SUCCEEDED(status)) 
	{
		/* Load user specified DSP Executable and start the processor. */
		if(fLoadDsp)
		status = InitializeProc(&pwrTask);
	}
	if (DSP_SUCCEEDED(status)) {
        /* Perform node level initialization. */
	argsBuf.cbData = strlen(argsBuf.cData) + 1;
	status = CreateNode(&pwrTask, &argsBuf);
        if (DSP_SUCCEEDED(status)) {
		status = RunNode(&pwrTask);
		if (DSP_SUCCEEDED(status)) 
		{
		if(fOpenLog)
			TSTLOG_appendLog(pHandle,  "RunTask Succedded\n");
		}
        }
    }
     DestroyNode(&pwrTask);
    if (pwrTask.hProcessor) 
	{
        status = DSPProcessor_Stop(pwrTask.hProcessor);
	if(!DSP_SUCCEEDED(status))
	{
		TSTLOG_appendLog(pHandle,  "DSPProcessor_Stop failed staus =0x%x\n",status);
	}
        DetachProcessor(&pwrTask);
        pwrTask.hProcessor = NULL;
    }

	if(DSP_SUCCEEDED(status))
	{
		TSTLOG_appendLog(pHandle, "TESTRESULT: PASS\n");
	}else{
		TSTLOG_appendLog(pHandle, "TESTRESULT: FAIL\n");
	}
	return (DSP_SUCCEEDED(status) ? 0 : -1);
}


/*
 *  ======== InitializeProc ========
 *  Perform processor related initialization.
 */
static DSP_STATUS InitializeProc(struct PWR_TASK *pwrTask)
{
	DSP_STATUS          status = DSP_SOK;
	INT		    iArgc = 1 ;

	status = DSPProcessor_Load(pwrTask->hProcessor, iArgc, aArgv, aEnvp);  
	if (DSP_SUCCEEDED(status)){
		if(fOpenLog)
			TSTLOG_appendLog(pHandle, "DSPProcessor_Load succeeded\n");
		status = DSPProcessor_Start(pwrTask->hProcessor);
		if (DSP_FAILED(status))	{
			if(fOpenLog)
			TSTLOG_appendLog(pHandle, "Error in DSPProcessor Start.Status = 0x%x\n", status);
		}
	}else{
		if(fOpenLog)
		TSTLOG_appendLog(pHandle,"Error in DSPProcessor_Load. Status = 0x%x\n",status);
	}
	return (status);
}

/*
 *  ======== CreateNode ======= *  Perform node related initialization.
 */
static DSP_STATUS CreateNode(struct PWR_TASK *pwrTask,
												struct PWR_NODEDATA *argsBuf)
{
	struct DSP_CBDATA *pArgs;
	DSP_STATUS status;

	pArgs = (struct DSP_CBDATA *)argsBuf;
	/* Allocate the ping node, passing arguments for node create phase. */
	status = DSPNode_Allocate(pwrTask->hProcessor, &PWR_TI_uuid, pArgs, NULL,
															&pwrTask->hNode);
	if (DSP_FAILED(status)) {
		fprintf(stdout, "DSPNode_Allocate failed: 0x%x\n", (UINT)status);
	}
	/* Create the ping node on the DSP. */
	if (DSP_SUCCEEDED(status)) {
		status = DSPNode_Create(pwrTask->hNode);
		if (DSP_FAILED(status)) {
			fprintf(stdout, "DSPNode_Create failed: 0x%x\n", (UINT)status);
		}
        else
           fprintf(stdout,"DSPNodeCreate succeeded \n"); 
    }
	/* Register for message notifications:  */
	if (DSP_SUCCEEDED(status)) {
		pwrTask->hEvent = malloc(sizeof(struct DSP_NOTIFICATION));
		status = DSPNode_RegisterNotify(pwrTask->hNode, DSP_NODEMESSAGEREADY, 
											DSP_SIGNALEVENT, pwrTask->hEvent);
		if (DSP_FAILED(status)) {
			fprintf(stdout,"DSPNode_RegisterNotify failed:0x%x\n",(UINT)status);
		}
        else
           fprintf(stdout,"DSPNode_registerNotify succeeded \n"); 
	}
	return (status);
}

/*
 *  ======== RunNode ======= *  Run ping task.
 */
static DSP_STATUS RunNode(struct PWR_TASK *pwrTask)
{
	DSP_STATUS status;
	DSP_STATUS exitStatus;
	struct DSP_MSG dspMsg;
	UINT uIndex;
	DWORD cEvents;
	struct DSP_NOTIFICATION* ahEvents[2];
	int n = 0;

	cEvents = 1;
	ahEvents[0] = pwrTask->hEvent;

	/* Start the ping node on the DSP. */
	status = DSPNode_Run(pwrTask->hNode);
	if (DSP_FAILED(status)) {
		fprintf(stdout, "DSPNode_Run failed: 0x%x\n", (UINT)status);
	} else
		fprintf(stdout,"DSPNode_run succeeded \n"); 

	if (DSP_SUCCEEDED(status)) {
		for (n = 0;n < pwrTask->msgCount;n = n + 1) {
	switch(pwrTask->Cmd){
	
		case 1:
		dspMsg.dwCmd = 1;
		break;

		case 2:
		dspMsg.dwCmd = 2;
		dspMsg.dwArg1 = pwrTask->Load;
		dspMsg.dwArg2 = pwrTask->minOpp;
		break;

		case 3:
		dspMsg.dwCmd = 3;
		dspMsg.dwArg1 = pwrTask->Load;
		dspMsg.dwArg2 = pwrTask->minOpp;
		break;

		case 4:
		dspMsg.dwCmd = 4;
		dspMsg.dwArg1 = pwrTask->minOpp;
		dspMsg.dwArg2 = pwrTask->maxOpp;
		break;

		default:
		fprintf(stdout,"Invalid Command for Testcase %d\n",pwrTask->Cmd);
		status =DSP_EFAIL;
		break;
	}

			status = DSPNode_PutMessage(pwrTask->hNode, &dspMsg, 15000);
			if (!DSP_SUCCEEDED(status)) {
				fprintf(stdout, "DSPNode_PutMessage failed, 0x%x.\n",
																(UINT)status);
			}
			/* Block until pinged by DSP or key event: */
			status = DSPManager_WaitForEvents(ahEvents, cEvents, &uIndex,
																DSP_FOREVER);
			/* Process events: */
			if (DSP_SUCCEEDED(status) && uIndex == 0) {
				/* DSP signalled us: */
				while (DSP_SUCCEEDED(status)) {
					status = DSPNode_GetMessage(pwrTask->hNode, &dspMsg, 0);
					if (DSP_SUCCEEDED(status)) {
						fprintf(stdout, "PWR: ReplyCmd %f Msg %f Mem %f\n",
									(DOUBLE)dspMsg.dwCmd, (DOUBLE)dspMsg.dwArg1,
														(DOUBLE)dspMsg.dwArg2);
					}
				}
				if (DSP_FAILED(status)) {
					if (status == DSP_ETIMEOUT) {
						/* Okay to timeout if message queue is empty */
						status = DSP_SOK;
						continue;
					} else {
						fprintf(stdout,"DSPNode_GetMessage failed: 0x%x\n",
																(UINT)status);
					}
				}
			} else {
				/* Wait error: */
				fprintf(stdout,"DSPManager_WaitForBridgeEvents Failed: 0x%x\n",
																(UINT)status);
			}
		}
	}
	/* Terminate the DSP node. */
	status = DSPNode_Terminate(pwrTask->hNode, &exitStatus);
	if (DSP_FAILED(status)) {
		fprintf(stdout, "DSPNode_Terminate failed: 0x%x\n",(UINT)status);
	}
	return (status);
}

/*
 *  ======== DestroyNode ======= *  Perform node related cleanup.
 */
static DSP_STATUS DestroyNode(struct PWR_TASK *pwrTask)
{
	DSP_STATUS status = DSP_SOK;

	if (pwrTask->hNode) {
		/* Delete DSP node. */
		status = DSPNode_Delete(pwrTask->hNode);
		if (DSP_FAILED(status)) {
			fprintf(stdout, "DSPNode_Delete failed: 0x%x\n", (UINT)status);
		}
		pwrTask->hNode = NULL;
	}
	if (pwrTask->hEvent) {
		free(pwrTask->hEvent);
	}
	return (status);
}

/*
 *  ======== DetachProcessor ======= *  Perform processor related cleanup.
 */
static DSP_STATUS DetachProcessor(struct PWR_TASK *pwrTask)
{
	DSP_STATUS status = DSP_SOK;

	if (pwrTask->hProcessor) {
		/* Detach from processor. */
		status = DSPProcessor_Detach(pwrTask->hProcessor);
		if (DSP_FAILED(status)) {
			fprintf(stdout, "DSPProcessor_Detach failed: 0x%x\n",(UINT)status);
		}
		pwrTask->hProcessor = NULL;
	}

	return (status);
}
