/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */

/* Hardcoded sample parameters: */
#define DEFAULTBUFSIZE  1024        /* Default buffer size in bytes. */
#define DEFAULTDSPUNIT  0           /* Default unit number for the board. */

#define DEFAULTNBUFS    1           /* Default # of buffers. */
#define NUMBUFS         2           /* Actual  # of buffers. */

#define STRMCOPY_TASK_ID TEXT("strmcopy")
#define ARGSIZE         32
#define DSPWORDSIZE     2           /* 1 DSP word = 2 bytes. */
#define NLOOPS          10
#define DEFAULTTIMEOUT  10000
#define DEFAULTALIGN    0
#define LOOPS           8

#define	DEFNAMESIZE     80
#define DEFAULTSEGMENT   0

#define DSP_SHMSEG0		-1

#define DSPEXEC         "inout_nortdxidl.dof"

static CHAR			    szLogName[DEFNAMESIZE];

static CONST  CHAR	    *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1"};
static BOOL			    fLoadDsp = FALSE, fOpenLog = FALSE;
static CONST  CHAR	    *aEnvp[DEFNAMESIZE] = {NULL};
static void			    *pHandle;

/* Stream copy task context data structure. */
typedef struct {
    DSP_HPROCESSOR      hProcessor;     /* Handle to processor. */
    struct DSP_UUID            hNodeID;        /* Handle to node ID GUID. */
    DSP_HNODE           hNode;          /* Handle to node. */
    DSP_HSTREAM         hOutStream;     /* Handle to node input stream. */
    DSP_HSTREAM         hInStream;      /* Handle to node output stream. */
    BYTE                **ppInBufs;     /* Input stream buffers. */
    BYTE                **ppOutBufs;    /* Output stream buffers. */
    struct DSP_NOTIFICATION    hNotification;  /* DSP notification object. */
    INT                 nStrmMode;      /* stream mode */
} STRMCOPY_TASK; 


typedef struct {
	UINT	iBufferSize;
	UINT	iSegmentId;
	UINT	iMode;
	UINT	iNumTxfrs;
}NODEARGS;


/* Forward declarations: */
static DSP_STATUS ProcessArgs(int argc, char ** argv);


/* Initialization and cleanup routines. */
static DSP_STATUS InitializeProcessor(STRMCOPY_TASK *copyTask);
static DSP_STATUS InitializeNode(STRMCOPY_TASK *copyTask);
static DSP_STATUS InitializeStreams(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupProcessor(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupNode(STRMCOPY_TASK *copyTask);
static DSP_STATUS CleanupStreams(STRMCOPY_TASK *copyTask);
static DSP_STATUS RunTask(STRMCOPY_TASK *copyTask);
static DSP_STATUS SendTestConfiguration(SHORT **pInBuf, 
							 SHORT **pOutBuf, 
							 STRMCOPY_TASK *copyTask, 
							 DWORD dwSendBufSize, 
							 DWORD dwRecvBufSize);


/* UUID's for PutGetData, PutData and GetData */
/* strmcopy node */

/* {7EEB2C7E-785A-11d4-A650-00C04F0C04F3} */
static const struct DSP_UUID IssueReclaimuuid = {
    0x7eeb2c7e, 0x785a, 0x11d4, 0xa6, 0x50, {
        0x0, 0xc0, 0x4f, 0xc, 0x4, 0xf3}
};

/* { FF6A1591_AFA1_11D6_8158_00E0B8142708 } */
static const struct DSP_UUID GetDatauuid = {
    0xFF6A1591, 0xAFA1, 0x11D6, 0x81, 0x58, {
        0x0, 0xE0, 0xB8, 0x14, 0x27, 0x08}
}; 


/* {F95CA076_EE7D_11D5_A8DD_00B0D055F6D1 } */
static const struct DSP_UUID inoutuuid = {	
	0xF95CA076, 0xEE7D, 0x11d5, 0xa8, 0xdd, {
		0x0, 0xb0, 0xd0, 0x55, 0xf6, 0xd1}
};
