/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    behvr.c
 *
 *   Path    $(TESTSUITEROOT)\Behaviour\behvr\
 * 
 *   Desc:
 *        Automated Environment for running GPP side Behaviour Tests 
 *        "behvr -v" will run all tests verbose.
 *        "behvr -l output.txt" will run all tests and store output to 
 *                   a file called "output.txt"
 *
 *  Usage:
 *        behvr [-v] [-l "log path/filename"] 
 *               [-x DSP executable ]
 *               [-m module to test (or ALL for all) ]
 *               [-n list of API functions to test for 
 *                   particular module or "ALL" for all functions ]
 *               [-# number of times ]
 *
 *        -v: verbose mode
 *        -l: use log and location to store results
 *        -x: DSP executable to be used in testing
 *        -m: individual module to test or ALL for all modules
 *            (Default = ALL)
 *        -n: list of API functions (see below) or ALL for all functions
 *            (Default = ALL)
 *        -#: number of times to run each API function test
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL 23, 2003    REF=                Swamy MC
 *      Added test cases for Dynamic Linking
 *   JUL 09, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>
#include <behvr.h>

INT dspnodest_main (INT , CHAR ** );
INT dspprocst_main (INT , CHAR ** );
INT dspstreamst_main (INT , CHAR ** );
INT doubleDspImageLoad (INT , CHAR ** );
INT mtFaultTest (INT , CHAR ** );
INT mmuFaultTest(INT , CHAR **);
INT mmuFaultTestMsg(INT , CHAR **);
INT flush_BEHAVIOUR (INT , CHAR ** );
INT pauseRun_BEHAVIOUR (INT , CHAR ** );
INT queueData_BEHAVIOUR (INT , CHAR ** );
INT multipleAttach_BEHAVIOUR (INT , CHAR ** );
INT multiThreadAttach_BEHAVIOUR (INT , CHAR ** );
INT waitForEvent_BEHAVIOUR (INT , CHAR ** );
INT nodeAttr_BEHAVIOUR (INT , CHAR ** );
INT oemConfig_BEHAVIOUR (INT , CHAR ** );
INT multiRetStop_BEHAVIOUR(INT , CHAR **);
INT multiStartRetStop_BEHAVIOUR(INT , CHAR **);
INT multiStartStop_BEHAVIOUR(INT , CHAR **);
INT multiStartRetStopNeg_BEHAVIOUR(INT , CHAR **);
INT multiNodeRun_BEHAVIOUR(INT , CHAR **);
INT TwlCheck(INT , CHAR **);
#ifndef __KERNEL__
INT dynFiveNodes_BEHAVIOUR (INT , CHAR ** );
INT dynFiveNodesSP_BEHAVIOUR (INT , CHAR ** );
INT depLibCheck_BEHAVIOUR (INT , CHAR ** );
INT depLibCheckSP_BEHAVIOUR (INT , CHAR ** );
INT memGranCheck_BEHAVIOUR (INT , CHAR ** );
INT memGranSPCheck_BEHAVIOUR (INT , CHAR ** );
INT dynLgMem_BEHAVIOUR (INT , CHAR ** );
INT dynLgMemSP_BEHAVIOUR (INT , CHAR ** );
INT dynFault_BEHAVIOUR (INT , CHAR ** );
INT dynUnrelBase_BEHAVIOUR (INT , CHAR ** );
#endif
INT regunregDynNodes_API (INT , CHAR ** );
INT PowerStateTest_main(INT , CHAR **);
INT ForceStateTest_main (INT ,CHAR **);
INT DvfsTest_main(INT, CHAR **);

/* 
 *  ======== behvr_main ======== 
 */
INT behvr_main (INT argc, CHAR * argv[])
{

    CHAR        * tmpSuiteName = "ALL";
    CHAR        ** tmp_argv;
    INT         tmp_argc;


    tmpSuiteName = argv[1];
    /* Now drop the first parameter and pass the rest of the
    parameters to corresponding Test Suite */
    tmp_argc = argc;  tmp_argc--;
    argv++;  tmp_argv = argv;

    if (!CSL_Strcmp(tmpSuiteName, "DOUBLEDSP"))
    {
        doubleDspImageLoad(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "NODEST"))
    {
        dspnodest_main(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PROCST"))
    {
        dspprocst_main(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "STREAMST"))
    {
        dspstreamst_main(tmp_argc, tmp_argv);
    }
#ifndef __KERNEL__
    else if (!CSL_Strcmp(tmpSuiteName, "MTFAULT"))
    {
        mtFaultTest(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MMUFAULT"))
    {
        mmuFaultTest(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MMUFAULTMSG"))
    {
        mmuFaultTestMsg(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PWRSTTEST"))
    {
        PowerStateTest_main(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "FORCESTTEST"))
    {
       ForceStateTest_main(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DVFSTEST"))
    {
	DvfsTest_main(tmp_argc, tmp_argv);
    }
    else if(!CSL_Strcmp(tmpSuiteName, "TWLCHECK"))
    {
	TwlCheck(tmp_argc, tmp_argv);
    }
#endif
    else if (!CSL_Strcmp(tmpSuiteName, "FLUSH"))
    {
        flush_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "PAUSERUN"))
    {
        pauseRun_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "QDATA"))
    {
        queueData_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULATTACH"))
    {
        multipleAttach_BEHAVIOUR(tmp_argc, tmp_argv);
    }
#ifndef __KERNEL__
    else if (!CSL_Strcmp(tmpSuiteName, "MULTHRDATTACH"))
    {
        multiThreadAttach_BEHAVIOUR(tmp_argc, tmp_argv);
    }
#endif
    else if (!CSL_Strcmp(tmpSuiteName, "EVENTWAIT"))
    {
        waitForEvent_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "NODEATTR"))
    {
        nodeAttr_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "OEMCONFIG"))
    {
        oemConfig_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTINODERUN"))
    {
	multiNodeRun_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTISTARTSTOP"))
    {
	multiStartStop_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTISTARTRETSTOPNEG"))
    {
	 multiStartRetStopNeg_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTISTARTRETSTOP"))
    {
	multiStartRetStop_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MULTIRETSTOP"))
    {
	multiRetStop_BEHAVIOUR(tmp_argc, tmp_argv);
    }


#ifndef __KERNEL__
    else if (!CSL_Strcmp(tmpSuiteName, "DYNFIVENODES"))
    {
        dynFiveNodes_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNFIVENODESSP"))
    {
        dynFiveNodesSP_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNDEPLIB"))
    {
        depLibCheck_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNDEPLIBSP"))
    {
        depLibCheckSP_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MEMSECTGRAN"))
    {
        memGranCheck_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "MEMSECTGRANSP"))
    {
        memGranSPCheck_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNLGMEM"))
    {
        dynLgMem_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNLGMEMSP"))
    {
        dynLgMemSP_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNFAULT"))
    {
        dynFault_BEHAVIOUR(tmp_argc, tmp_argv);
    }
    else if (!CSL_Strcmp(tmpSuiteName, "DYNUNRELBASE"))
    {
        dynUnrelBase_BEHAVIOUR(tmp_argc, tmp_argv);
    }
#endif
    else if (!CSL_Strcmp(tmpSuiteName, "REGUNREG"))
    {
        TST_verbose("Calling REGUNREG nodes \n");
        regunregDynNodes_API(tmp_argc, tmp_argv);
    }

    else
    {
        TST_verbose("Invalid test suite specified: %s\n", tmpSuiteName);
    }

    return 0;
}


