/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    multiProc.c
 *
 *   Path    $(TESTSUITEROOT)\Stress\MultiProcess\proc\multiProc.c
 * 
 *   Desc:
 *           multiProc is called from the Multi Process Stress
 *           test suite and checks for Data transfer. Multiple instances of
 *           this process will be invoked by the Stress Test case.
 *
 *  Usage: Called from MultiProcessData_STRESS. Check that function
 *  for more details.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2002
 *
 *   ============================================================================
 *   Revision History
 *
 *   AUG 15, 2002    REF=ORG             Swamy MC
 *      Original Version.
 *   ============================================================================
 */


#include <stdio.h>
#include <dbapi.h>
#include <getopt.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#include "MultiProcessData.h"

#define DEFNAMESIZE      100
#define NUMOFCOMPARGS      3	/* Compulsory args: t,n,s */
#define MAXNUMOFPROCS      6    /* This number taken from Symbian Test Suite */

#define SHORT short


/* Function to exit gracefully */
INT graceful_exit()
{
	printf("FATAL ERROR: Exiting the Test Case !\n");
	exit(0);
}


INT main(INT argc, CHAR * argv[])
{

    struct DSP_STRMATTR			attrs;
	BYTE					**arrBuf;
    struct DSP_NODEATTRIN			nodeAttrin;
    DSP_HNODE				hNode = NULL;
	DSP_HSTREAM				hStream;
	CHAR					*pszUuid;
	struct DSP_UUID				gidNodeID;
    struct DSP_CBDATA				*cbData;
	DSP_STATUS				status;
	INT						i;
	DSP_STATUS				exitStatus = DSP_SOK;

	/* Processor related information */
	DSP_HPROCESSOR			hProcessor;
	struct DSP_PROCESSORATTRIN         *pAttrIn;

	/* Test case specific information */
	INT						iNumTransfers;
	INT						iBuffSize;

	/* Logging related information */
	FILE					*pHandle;
	BOOL					bUseLog=FALSE;
	BOOL					bVerbose=FALSE;


	/* Copy the contents of passed args to local vars */
	/* This strategy can be changed if the variables are just
	   used/read and not modified. (Swamy) */
/*	hProcessor = targs->hProcessor;
	iNumTransfers = targs->iNumTransfers;
	iBuffSize = targs->iBuffSize;
	pHandle = targs->pHandle;
	bUseLog = targs->bUseLog;
	bVerbose = targs->bVerbose;
 */


    /* Initialising attributes */
    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = 5000;

//	arrBuf = LocalAlloc(LMEM_FIXED,sizeof(BYTE *));
    arrBuf = OsSpecificMemAlloc(sizeof(BYTE *));


    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
	if (pAttrIn == NULL)
	{
		// Error
		TST_verbose("Malloc for pAttrIn FAILED !!\n");
		graceful_exit();
	}
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    /* Attach to Processor */
    status = DSPProcessor_Attach(0, pAttrIn, &hProcessor);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,
            "Error in DSPProcessor_Attach()\n");
        }
        graceful_exit();
    }

#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif


	pszUuid = "F95CA07A_EE7D_11D5_A8DD_00B0D055F6D1";  // taken from mttask_nortdx_int.cdb file
    UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

    /* Allocate Node */
    status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");
        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Allocate().\n");
        }
        graceful_exit();
    }

    status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        graceful_exit();
    }

    status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                             &attrs);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Connect()\n");
        }
        graceful_exit();
    }


    /* Create Node on DSP */
    status = DSPNode_Create(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Create()\n");
        }
        graceful_exit();
    }

    status = DSPNode_Run(hNode);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPNode_Run()\n");
        }
        graceful_exit();
    }

    status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &hStream);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStraem_Open()\n");
        }
        graceful_exit();
    }

    status = DSPStream_AllocateBuffers(hStream, 64, arrBuf, 1);

    if (!DSP_SUCCEEDED(status)) 
	{
        /* Report Error */
        TST_verbose("FAIL\n");

        if (bUseLog) 
		{
            TSTLOG_appendLog(pHandle, "FAIL\n");
            TSTLOG_appendLog(pHandle,"Error in DSPStream_AllocateBuffers()\n");
        }
        graceful_exit();
    }

    /* fill buffer */
    for (i = 0; i < iBuffSize; i++) 
	{
        (*(arrBuf))[i] = (BYTE) 'a';
    }

	/* Do the Data Transfer */


        /* fill the remaining buffer 
        for (i = 0; i < ((iBuffSize / 2) - 5); i++) 
	    {
            *arrBuf[0]++ = (SHORT) i;
			arrBuf[0]++;
        }
		*/


        /*  Data transfer for number of iterations .. specified by the
         *  user as parameters to the test ... set in the constructor
	     */
        for (i = 0 ; i < iNumTransfers ; i++)
	    {
			/* Issue Data */
			status = DSPStream_Issue(hStream, arrBuf[0], 64, 64, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Issue()\n");
				}
				graceful_exit();
			}

			/* Reclaim Data */
			status = DSPStream_Reclaim(hStream, arrBuf[0], 64, 64, 0);

		    if (!DSP_SUCCEEDED(status)) 
			{
		        /* Report Error */
		        TST_verbose("FAIL\n");
	
				if (bUseLog) 
				{
					TSTLOG_appendLog(pHandle, "FAIL\n");
					TSTLOG_appendLog(pHandle,"Error in DSPStraem_Reclaim()\n");
				}
				graceful_exit();
			}

			/* Check for validity of Data */

	    } // End of For loop - iNumTransfers


    status = DSPStream_Close(hStream);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);

    OsSpecificMemFree(arrBuf);
    OsSpecificMemFree(pAttrIn);

	/* Exit from this Process */
	exit(0);
}



