/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_GetResourceInfo.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_GetResourceInfo.c
 * 
 *   Desc:
 *      DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *      DSPProcessor_GetResourceInfo(DSP_HPROCESSOR *phProcessor,
 *                                   UINT uResourceType,
 *                                   DSP_RESOURCEINFO *pResourceInfo,
 *                                   UINT uResourceInfoSize);
 *  Usage:
 *        DSPProcessor_GetResourceInfo [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>


#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC        "dataxfr_nortdxidl.dof"


/*
 *  ======== main ========
 *
 */
INT DSPProcessor_GetResourceInfo_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT                 opt;
    INT                 cRetVal = 1;
    INT                 i, j, k, l;
    BOOL                bUseLog = FALSE;
    FILE                * pHandle=NULL;
    CHAR                szLogName[DEFNAMESIZE];
    DSP_STATUS          status;
    struct DSP_PROCESSORATTRIN * pAttrIn;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] = /* arg1: handle of attached processor */
    {
        NULL,           /* Valid handle to be attached */
        NULL,           /* Invalid NULL handle */
    };

    /* INVALID numeric resource types are not detected */
    UINT arg2[] =   /* arg2: uResourceType */
    {
        DSP_DYNDARAM,   /* valid type */
        0x2492          /* invalid type */
    };

    struct DSP_RESOURCEINFO *arg3[] = /* Resource Info Pointer */
    {
        NULL,           /* Valid pointer to be set */
        NULL            /* Invalid NULL pointer */
    };

    /* INVALID size is not detected anymore since sizes can vary */
    UINT arg4[] =   /* arg4: ResourceInfo size */
    {
        sizeof(struct DSP_RESOURCEINFO),   /* valid size */
        0                           /* valid size */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))
    #define NUMARG2 (sizeof(arg2)/sizeof(UINT))
    #define NUMARG3 (sizeof(arg3)/sizeof(struct DSP_RESOURCEINFO *))
    #define NUMARG4 (sizeof(arg4)/sizeof(UINT))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][0][0][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][0][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][1][1] */
                },
            },
            {
                {
                    {DSP_EVALUE, TST_SHOULDEQUAL},      /* [0][1][0][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},      /* [0][1][0][1] */
                },
                {
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][1][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][1][1][1] */
                },
            },
        },
        {
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][0][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][0][1][1] */
                },
            },
            {
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][1][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][1][0][1] */
               },
               {
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][1][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL},     /* [1][1][1][1] */
               },
            },
        },
    };

    /* Allocate pointer to dummy set of attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    arg3[0] = OsSpecificMemAlloc(sizeof(struct DSP_RESOURCEINFO));

    /* Reset getopt pointer */
    optind = 0;

    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPProcessor_GetResourceInfo: Verbose Mode On\n");
            break;
            
        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            
            TSTLOG_appendLog(pHandle, "DSPProcessor_GetResourceInfo:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;

        case 'x':
            /* Use DSP executable */
            
            /*
             *  Not used in this test - SKIP
             *  Leave option here, however, in case executable argument
             *  is passed in from 'apitst'
             */
            break;

        default:
            TST_verbose(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle,
                    "Invalid option specified.\n");
            }
            break;
        }
    }

    TST_verbose("DSPProcessor_GetResourceInfo: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
       if (!DSP_SUCCEEDED(status)) {
           /* Report Error */
           cRetVal = 0;
           if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPProcessor_Attach()\n");
           }
           break;
       }

       for (i = 0; i < NUMARG1; i++) {
           for (j = 0; j < NUMARG2; j++) {
               for (k = 0; k < NUMARG3; k++) {
                   for (l = 0; l < NUMARG4; l++) {
                       status = DSPProcessor_GetResourceInfo(arg1[i],
                                          arg2[j], arg3[k], arg4[l]);

                       if (!TST_CheckVal(status,
                           checkArray[i][j][k][l].dwValue,
                           checkArray[i][j][k][l].fEqualFlag)) {
                           cRetVal = 0;
                           TST_verbose("GetResourceInfo "
                               "[%d][%d][%d][%d] returned %x\n",
                               i, j, k, l, status);
                           if (bUseLog) {
                               TSTLOG_appendLog(pHandle, "FAIL\n");
                               TSTLOG_appendLog(pHandle,
                                   "DSPProcessor_GetResourceInfo "
                                   "[%d][%d][%d][%d] failed.\n", 
                                   i, j, k, l);
                           }
                       }
                   }
               }
           }
       }

       break;
    }

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    status = DSPProcessor_Detach(arg1[0]);

    OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(arg3[0]);

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

    return 0;
}

