/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPProcessor_InvalidateMemory.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPProcessor\DSPProcessor_InvalidateMemory.c
 * 
 *   Desc:
 *        DSPProcessor API test
 *
 *  DSPProcessor API function tested:
 *  	  DSPProcessor_InvalidateMemory(DSP_HPROCESSOR hProcessor, PVOID pMpuAddr,
 * 			 ULONG ulSize)
 *  Usage:
 *        DSPProcessor_FlushMemory [-v] [-l "log path/filename"]
 *
 *        -v: verbose mode
 *        -l: use log and location
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <testosal.h>


#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
/*
 *  ======== main ========
 *
 */
INT DSPProcessor_InvalidateMemory_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT            opt;
    INT            cRetVal = 1;
    INT            i;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    DSP_STATUS        status;
    struct DSP_PROCESSORATTRIN *pAttrIn;
    struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId=0;
    ULONG	pMpuBuffer=0;
    ULONG       BufferSize = 0x19000;


    /* INPUT PARAMETERS */
    DSP_HPROCESSOR arg1[] =     /* arg1: handle of attached processor */
    {
        NULL, /* Valid handle to be attached */
        NULL, /* Invalid NULL handle */
    };



    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HPROCESSOR))

    /* SOLUTION SET */
    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1]  =
    {
                {DSP_SOK, TST_SHOULDEQUAL}, /* [0] */
                {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1] */
    };


    /* Reset getopt pointer */
    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
         switch (opt) {
         case 'v':
             /* verbose */
             TST_setVerbose(TRUE);
             TST_verbose("DSPProcessor_InvalidateMemory: Verbose Mode On\n");
             break;
      case 'l':
             /* use log and store with name location */
             strncpy(szLogName, optarg, sizeof(szLogName));
          bUseLog = TRUE;
             TST_verbose("Opening %s\n", szLogName);
             pHandle = TSTLOG_openLog(szLogName);
             TSTLOG_appendLog(pHandle, "DSPProcessor_InvalidateMemory:\n");
             /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
             TST_verbose("Log opened as: %s\n", szLogName);
          break;
         case 'x':
             /* Use DSP executable */

             /*
              *  Not used in this test - SKIP
              *  Leave option here, however, in case executable argument
              *  is passed in from 'apitst'
              */

             break;
         default:
             TST_verbose(("Invalid option: %c\n"), opt);
             cRetVal = 0;
             if (bUseLog) {
                 TSTLOG_appendLog(pHandle,
                 "Invalid option specified.\n");
             }
             break;
         }
    }

 /* Allocate pointer to dummy set of attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    TST_verbose("DSPProcessor_FlushMemory: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &arg1[0]);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()\n");
            }
            break;
        }
       pMpuBuffer = (ULONG) OsSpecificMemAlloc(BufferSize);

	/*Intialize buffers*/

	 for (i = 0; i < BufferSize; i++) {
                ((PSTR) pMpuBuffer)[i] = 0;
        }


        for (i = 0; i < NUMARG1; i++) {
                    status = DSPProcessor_InvalidateMemory(arg1[i], (PVOID)pMpuBuffer, BufferSize);

                    if (!TST_CheckVal(status,
                        checkArray[i].dwValue,
                        checkArray[i].fEqualFlag)) {
                        cRetVal = 0;
                        TST_verbose("InvalidateMemory [%d] %x\n",
                                    i, status);

                        if (bUseLog) {
                            TSTLOG_appendLog(pHandle, "FAIL\n");
                            TSTLOG_appendLog(pHandle,
                            "DSPProcessor_FlushMemoryl \
                            [%d] failed.\n", i);
                        }
                    }
                }

        /* Detach Processor */
        status = DSPProcessor_Detach(arg1[0]);

        break;
    }

    OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(pMpuBuffer);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }
    return 0;
}

