/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    DSPStream_Idle.c
 *
 *   Path    $(TESTSUITEROOT)\TestAPI\DSPStream\DSPStream_Idle.c
 * 
 *   Desc:
 *        DSPStream_Idle API test
 *
 *  DSPStream API function tested:
 *        DSPStream_Idle(DSP_HSTREAM hStream, BOOL bFlush)
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <DSPProcessor_OEM.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>

#define DEFNAMESIZE    80
#define DEFAULTDSPUNIT 0
#define DSPEXEC        "dataxfr_nortdxidl.dof"

/*
 *  ======== main ========
 *
 */
INT DSPStream_Idle_API (INT argc, CHAR * argv[], INT iRunTime)
{
    INT            opt;
    INT            cRetVal = 1;
    INT            i, j;
    INT            iArgc = 1;
    BOOL        bUseLog=FALSE;
    FILE        *pHandle=NULL;
    CHAR        szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                *pszUuid;
    DSP_HPROCESSOR     hProcessor;
    DSP_HNODE        hNode = NULL;
    DSP_STATUS        status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_PROCESSORATTRIN    *pAttrIn;
    struct DSP_UUID            gidNodeID;
    struct DSP_CBDATA          *cbData;
    struct DSP_NODEATTRIN      nodeAttrin;
    struct DSP_STRMATTR        attrs;
    BYTE                argsBuf[32 + sizeof(ULONG)];


    /* INPUT PARAMETERS */
    DSP_HSTREAM arg1[] =     /* arg 1: handle of a stream */
    {
        NULL, /* valid handle to be allocated */
        NULL                    /* invalid NULL handle */
    };

    BOOL arg2[] =            /* arg 2: boolean flag */
    {
        TRUE, /* valid flag */
        TRUE                    /* valid flag */
    };

    #define NUMARG1 (sizeof(arg1)/sizeof(DSP_HSTREAM))
    #define NUMARG2 (sizeof(arg2)/sizeof(BOOL))

    /* SOLUTION SET */
    struct  {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] =
    {
        {
            {DSP_SOK, TST_SHOULDEQUAL}, /* [0]  [0] */
            {DSP_SOK, TST_SHOULDEQUAL}, /* [0]  [1] */
        },
        {
            {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1]  [0] */
            {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1]  [1] */
        },
    };

    /* Allocate pointer to dummy attributes */
    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));

    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    nodeAttrin.uTimeout = 1000;
    nodeAttrin.iPriority = 1;
#if defined(OMAP_2430)|| defined(OMAP_3430)
    nodeAttrin.uProfileID = 0xff;
#endif



    cbData = (struct DSP_CBDATA *)argsBuf;
    cbData->cbData = 32;

    attrs.uSegid = 0;
    attrs.uBufsize = 512;
    attrs.lMode = STRMMODE_PROCCOPY;
    attrs.uNumBufs = 1;
    attrs.uAlignment = 0;
    attrs.uTimeout = DSP_FOREVER;

    optind = 0;
    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:")) != EOF) {
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPStream_Idle: Verbose Mode On\n");
            break;
       case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
         bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPStream_Idle:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
         break;
    case 'x':
        /* use given executable */
        aArgv[0] = (char *) optarg;
        break;
        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;

            if (bUseLog) {
               TSTLOG_appendLog(pHandle,
               "Invalid Option Specified.\n");
            }
            break;
        }
    }
    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPStream_Idle: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to processor */
        struct DSP_PROCESSORINFO       dspInfo;
        UINT                    numProcs;
        UINT                    index=0;
        INT                     procId=0;
    
        while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo, (UINT) sizeof (struct DSP_PROCESSORINFO),&numProcs))) {
            if (dspInfo.uProcessorType == DSPTYPE_55 || dspInfo.uProcessorType == DSPTYPE_64) {
                TSTLOG_appendLog(pHandle,"DSP device detected !! \n");
                procId = index;
                status = DSP_SOK;
                break;
            }
            index++;
        }

        /* Attach to an available DSP (in this case, the 1st DSP). */
        if (DSP_SUCCEEDED(status)) {
        
            status = DSPProcessor_Attach(procId, pAttrIn, &hProcessor);
            if (DSP_SUCCEEDED(status)) {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach succeeded.\n");
            }
            else {
                TSTLOG_appendLog(pHandle, "DSPProcessor_Attach failed. Status = 0x%x\n",status);
            }
        }
        else {
            TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");
        }
    
        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Attach()\n");
             }
            break;
        }

        arg1[1] = arg1[0];
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, 
						NULL);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load()\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start()\n");
            }
            break;
        }

        /*pszUuid = "B1D5C1A0_B750_11D4_A683_00C04F0C04F3";*/
        pszUuid = "F95CA072_EE7D_11D5_A8DD_00B0D055F6D1";//dataxfr
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, &nodeAttrin,
                   &hNode);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Allocate().\n");
            }
            break;
        }

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, hNode, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(hNode, 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }


        /* Create Node on DSP */
        status = DSPNode_Create(hNode);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Create()\n");
            }
            break;
        }

        status = DSPNode_Run(hNode);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Run()\n");
            }
            break;
        }

        status = DSPStream_Open(hNode, DSP_TONODE, 0, NULL, &arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
               TSTLOG_appendLog(pHandle, "FAIL\n");
               TSTLOG_appendLog(pHandle,
               "Error in DSPStream_Open()\n");
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {
            for (j = 0; j < NUMARG2; j++) {

         status = DSPStream_Idle(arg1[i], arg2[j]);

                if (!TST_CheckVal(status, checkArray[i][j].dwValue,
                    checkArray[i][j].fEqualFlag)) {
                    cRetVal = 0;
                    TST_verbose("Idle [%d][%d] returned %x\n", i, j, status);

                    if (bUseLog) {
                        TSTLOG_appendLog(pHandle, "FAIL\n");
                        TSTLOG_appendLog(pHandle,
                        "DSPStream_Idle [%d][%d] \
                        failed.\n", i, j);
                    }
                }
            }
        }

        break;
    }

    OsSpecificMemFree(pAttrIn);


    status = DSPStream_Close(arg1[0]);
    status = DSPNode_Terminate(hNode, &exitStatus);
    status = DSPNode_Delete(hNode);
    status = DSPProcessor_Detach(hProcessor);

    if (cRetVal) {
        /* Report in LOG module */
        if (bUseLog) {
            TSTLOG_appendLog(pHandle, "PASS\n");
        }
        TST_verbose("PASS\n");
    }

    /* Close Log */
    if (bUseLog) {
        TSTLOG_closeLog(pHandle);
    }

return 0;
}

