/*  ============================================================================
 *   File    IVADSPNode_AllocMsgBuf1.c
 *
 *   Path    $(TESTSUITEROOT)\TestIVAAPI\IVADSPNode
 * 
 *   Desc:
 *      DSPNode API test.  Test DSPNode_AllocMsgBuf with different
 *      allocation sizes and memory attributes.
 *
 *  DSPNode API function tested:
 *      DSPNode_AllocMsgBuf(DSP_HNODE hNode, UINT iSize,
 *                          DSP_BUFFERATTR *pAttr,
 *                          BYTE *pBuffer)
 *
 *  Usage:
 *      DSPNode_AllocMsgBuf [-v] [-l "log path/filename"]
 *                     [-x "valid executable"]
 *                     [-m "message to be sent"]
 *
 *      -v: verbose mode
 *      -l: use log and location
 *      -x: name of valid DSP executable
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *   AUG 05, 2004    REF=Based on DSP API tests             Ramesh Gupta G
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


#include <stdio.h>
#include <string.h>
#include <dbapi.h>
#include <getopt_test.h>
#include <tst.h>
#include <tstlog.h>
#include <uuidutil.h>
#include <testosal.h>
#include <DSPProcessor_OEM.h>
#include <msgtxfr.h>
#define DEFNAMESIZE    80
#define MAXBUF 0xFFF

#define DSPMESSAGE     0x1100

#define DSPEXEC         "iva.dof"

void ivaoutStatus(UINT);

/*
 *  ======== main ========
 */
INT IVADSPNode_AllocMsgBuf1_API (INT argc, CHAR * argv[], INT iRunTime)
{

    INT                 opt;
    INT                 cRetVal = 1;
    INT                 i, j, k, l;
    INT                 iArgc = 1;
    BOOL                bUseLog=FALSE;
    FILE                * pHandle=NULL;
    CHAR                szLogName[DEFNAMESIZE];
    CHAR                *aArgv[DEFNAMESIZE] = { DSPEXEC, "arg1" };
    CHAR                *pszUuid;
    DSP_HPROCESSOR      hProcessor;
    struct DSP_UUID            gidNodeID;
    DSP_STATUS          status;
    DSP_STATUS          exitStatus = DSP_SOK;
    struct DSP_MSG             pMessage;
    struct DSP_PROCESSORATTRIN * pAttrIn;
    struct DSP_NODEATTRIN      * nodeAttrin;
    struct DSP_CBDATA          * cbData;
    struct DSP_STRMATTR	attrs;
    struct struct DSP_PROCESSORINFO       dspInfo;
    UINT                    numProcs;
    UINT                    index=0;
    INT                     procId;

    /* INPUT PARAMETERS */
    DSP_HNODE arg1[] =  /* arg1: handle of node */
    {
        NULL,           /* Valid handle to be allocated */
        NULL            /* Invalid NULL handle */
    };

    UINT arg2[] =       /* arg2: buffer size */
    {
        64,             /* valid size */
        0,              /* invalid size */
        0xffffffff,     /* too big */
        MAXBUF,         /* maxsize (what is the max size defined here) */
        MAXBUF+1,       /* maxsize + 1 (what is the max size defined here) */
        MAXBUF-1        /* maxsize - 1 (what is the max size defined here) */
    };

    struct DSP_BUFFERATTR *arg3[] =    /* arg3: pointer to structure */
    {
        NULL,           /* Valid segment, valid alignment */
        NULL,           /* Valid segment, invalid alignment */
        NULL,           /* Invalid segment, valid alignment */
        NULL,           /* Invalid segment, invalid alignment */
        NULL            /* Valid (Default) NULL pointer */
    };

    BYTE **arg4[] =     /* arg4: pointer to allocated buffer */
    {
        NULL,           /* valid pointer to be allocated */
        NULL            /* invalid pointer */
    };

#define NUMARG1 (sizeof(arg1)/sizeof(DSP_HNODE))
#define NUMARG2 (sizeof(arg2)/sizeof(UINT))
#define NUMARG3 (sizeof(arg3)/sizeof(struct DSP_BUFFERATTR *))
#define NUMARG4 (sizeof(arg4)/sizeof(BYTE *))

    /* SOLUTION SET */
    BOOL validSizeArray[NUMARG2] = {TRUE, FALSE, FALSE, TRUE, FALSE, TRUE};

    struct {
        DWORD dwValue;
        BOOL fEqualFlag;
    } checkArray[NUMARG1] [NUMARG2] [NUMARG3] [NUMARG4] =
    {
        {
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][0][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][0][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][0][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][1][1] */
                },
                {
                    {DSP_EBADSEGID, TST_SHOULDEQUAL},   /* [0][0][2][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][2][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][0][3][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][3][1] */
                },
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][0][4][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][0][4][1] */
                },
            },
            {
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][0][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][0][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][1][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][1][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][2][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][2][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][3][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][3][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][4][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},       /* [0][1][4][1] */
                },
            },
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][2][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][2][0][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][2][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][2][1][1] */
                },
                {
                    {DSP_EBADSEGID, TST_SHOULDEQUAL},   /* [0][2][2][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][2][2][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][2][3][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][2][3][1] */
                },
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][2][4][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][2][4][1] */
                },
            },
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][3][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][3][0][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][3][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][3][1][1] */
                },
                {
                    {DSP_EBADSEGID, TST_SHOULDEQUAL},   /* [0][3][2][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][3][2][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][3][3][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][3][3][1] */
                },
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][3][4][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][3][4][1] */
                },
            },
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][4][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][4][0][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][4][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][4][1][1] */
                },
                {
                    {DSP_EBADSEGID, TST_SHOULDEQUAL},   /* [0][4][2][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][4][2][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][4][3][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][4][3][1] */
                },
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][4][4][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][4][4][1] */
                },
            },
            {
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][5][0][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][5][0][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][5][1][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][5][1][1] */
                },
                {
                    {DSP_EBADSEGID, TST_SHOULDEQUAL},   /* [0][5][2][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][5][2][1] */
                },
                {
                    {DSP_EALIGNMENT, TST_SHOULDEQUAL},  /* [0][5][3][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][5][3][1] */
                },
                {
                    {DSP_SOK, TST_SHOULDEQUAL},         /* [0][5][4][0] */
                    {DSP_EPOINTER, TST_SHOULDEQUAL},    /* [0][5][4][1] */
                },
            },
        },


        {
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][1][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][2][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][2][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][3][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][3][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][4][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][0][4][1] */
                },
            },
         {
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][0][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][0][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][1][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][1][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][2][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][2][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][3][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][3][1] */
                },
                {
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][4][0] */
                    {DSP_ESIZE, TST_SHOULDEQUAL},   /* [1][1][4][1] */
                },
            },
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][1][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][2][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][2][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][3][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][3][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][4][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][2][4][1] */
                },
            },
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][1][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][2][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][2][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][3][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][3][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][4][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][3][4][1] */
                },
            },
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][1][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][2][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][2][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][3][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][3][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][4][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][4][4][1] */
                },
            },
            {
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][0][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][0][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][1][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][1][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][2][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][2][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][3][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][3][1] */
                },
                {
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][4][0] */
                    {DSP_EHANDLE, TST_SHOULDEQUAL}, /* [1][5][4][1] */
                },
            },
        },
    };

    /* Initialise pHandle */
    pHandle = (FILE *)NULL;

    pAttrIn = OsSpecificMemAlloc(sizeof(struct DSP_PROCESSORATTRIN));
    pAttrIn->cbStruct = sizeof(struct DSP_PROCESSORATTRIN);
    pAttrIn->uTimeout = 50;

    
	arg3[0] = OsSpecificMemAlloc(sizeof(struct DSP_BUFFERATTR));
    arg3[0]->cbStruct = sizeof(struct DSP_BUFFERATTR);
    arg3[0]->uAlignment = 1;
    arg3[0]->uSegment = 1;

    
	arg3[1] = OsSpecificMemAlloc(sizeof(struct DSP_BUFFERATTR));
    arg3[1]->cbStruct = sizeof(struct DSP_BUFFERATTR);
    arg3[1]->uAlignment = 5;
    arg3[1]->uSegment = 1;

    
	arg3[2] = OsSpecificMemAlloc(sizeof(struct DSP_BUFFERATTR));
    arg3[2]->cbStruct = sizeof(struct DSP_BUFFERATTR);
    arg3[2]->uAlignment = 0;
    arg3[2]->uSegment = 3;

    
	arg3[3] = OsSpecificMemAlloc(sizeof(struct DSP_BUFFERATTR));
    arg3[3]->cbStruct = sizeof(struct DSP_BUFFERATTR);
    arg3[3]->uAlignment = 5;
    arg3[3]->uSegment = 3;

    /* initialise some more values here */
    
	arg4[0] = OsSpecificMemAlloc(sizeof(BYTE *));
    arg4[1] = NULL;

    
	nodeAttrin = OsSpecificMemAlloc(sizeof(struct DSP_BUFFERATTR));
    nodeAttrin->cbStruct = sizeof(struct DSP_NODEATTRIN);
    nodeAttrin->uTimeout = 2000;
    nodeAttrin->iPriority = 1;

    
	cbData = OsSpecificMemAlloc(sizeof(struct DSP_CBDATA));
    cbData->cbData = 1;

    pMessage.dwCmd = (DWORD)DSPMESSAGE;

     /* Reset getopt pointer */
    optind = 0;

    while ((opt = getopt_test(argc, (char *const)argv, "+vl:u:x:m:")) != EOF) {
        
        switch (opt) {
        case 'v':
            /* verbose */
            TST_setVerbose(TRUE);
            TST_verbose("DSPNode_AllocMsgBuf: Verbose Mode On\n");
            break;

        case 'l':
            /* use log and store with name location */
            strncpy(szLogName, optarg, sizeof(szLogName));
            bUseLog = TRUE;
            TST_verbose("Opening %s\n", szLogName);
            pHandle = TSTLOG_openLog(szLogName);
            TSTLOG_appendLog(pHandle, "DSPNode07:\n");
            /*TSTLOG_appendLog(pHandle, "Run #%i:", iRunTime);*/
            TST_verbose("Log opened as: %s\n", szLogName);
            break;

        case 'x':
            /* use given executable */
            aArgv[0] = (char *) optarg;
            break;

        case 'm':
            /* use given message to send to node */
            pMessage.dwCmd = (DWORD)optarg;
            break;

        default:
            TST_reportError(("Invalid option: %c\n"), opt);
            cRetVal = 0;
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "Invalid option specified.\n");
            }
            
            break;
        }
    }

    TST_verbose("DSP Executable: %s\n", aArgv[0]);
    TST_verbose("DSPNode_AllocMsgBuf: ");
    TST_verbose("%i: ", iRunTime);

    while (cRetVal > 0) {
        /* Attach to Processor */

/////////////////////////////////////////////////////////////////
 while (DSP_SUCCEEDED(DSPManager_EnumProcessorInfo(index,&dspInfo,(UINT) sizeof (struct struct DSP_PROCESSORINFO),&numProcs)))
        {
                if (dspInfo.uProcessorType == IVA_ARM7)
                {
                        TSTLOG_appendLog(pHandle,"IVA device detected !! \n");
                        procId = index;
                        status = DSP_SOK;
                        break;
                }
                index++;
        }

    /* Attach to IVA */
    if (DSP_SUCCEEDED(status)) {
        status = DSPProcessor_Attach(procId, NULL,&hProcessor);
        if (DSP_SUCCEEDED(status)) {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach succeeded.\n");
        }
        else {
            TSTLOG_appendLog(pHandle, "IVA:DSPProcessor_Attach failed. Status = 0x%x\n",status);
        }
    }
        else
           TSTLOG_appendLog(pHandle, "Failed to get the desired processor \n");

////////////////////////////////////////////////////////////////

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                    "Error in DSPProcessor_Attach():IVA \n");
            }
            break;
        }

        arg1[1] = arg1[0];
        /* Load default program */
        status = DSPProcessor_Load(hProcessor, iArgc, (CONST CHAR **)aArgv, NULL);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Load():IVA\n");
            }
            break;
        }

        status = DSPProcessor_Start(hProcessor);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPProcessor_Start():IVA\n");
            }
            break;
        }

		pszUuid = "6562DF2A_7068_42EF_8459_75584C1CDB8F";
        UUID_UuidFromString((CHAR *)pszUuid, &gidNodeID);
	gidNodeID=Iva_Msguuid;

        /* Allocate Node */
        status = DSPNode_Allocate(hProcessor, &gidNodeID, cbData, nodeAttrin,
                   &arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;

            TST_verbose("FAIL\n");
            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Allocate():IVA.\n");
            }
            break;
        }

		/* The following two Connect calls are added to accomodate the
		   usage of dataxfr dof instead of the apitest0 dof. The DSP
		   side expects the Connect calls. So these are added here */
/*		attrs.uSegid = 0;
		attrs.uBufsize = 512;
		attrs.lMode = STRMMODE_PROCCOPY;
		attrs.uNumBufs = 1;
		attrs.uAlignment = 0;
		attrs.uTimeout = 5000;

        status = DSPNode_Connect((DSP_HNODE)DSP_HGPPNODE, 0, arg1[0], 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }

        status = DSPNode_Connect(arg1[0], 0, (DSP_HNODE)DSP_HGPPNODE, 0,
                                 &attrs);

        if (!DSP_SUCCEEDED(status)) {
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Connect()\n");
            }
            break;
        }
/*

        /* Create Node on DSP */
        status = DSPNode_Create(arg1[0]);

        if (!DSP_SUCCEEDED(status)) {
            /* Report Error */
            cRetVal = 0;
            TST_verbose("FAIL\n");

            if (bUseLog) {
                TSTLOG_appendLog(pHandle, "FAIL\n");
                TSTLOG_appendLog(pHandle,
                "Error in DSPNode_Create()\n");
            }
            break;
        }

        for (i = 0; i < NUMARG1; i++) {

            for (j = 0; j < NUMARG2; j++) {

                for (k = 0; k < NUMARG3; k++) {

                    for (l = 0; l < NUMARG4; l++) {

                        status = DSPNode_AllocMsgBuf(arg1[i], arg2[j],
                            arg3[k], (BYTE**)arg4[l]);
                        
                        /* check the results make sense */
                        if (arg4[l] != NULL) {
                            /* out of memory */
                            if (DSP_SUCCEEDED(status) && (*arg4[l]) == NULL ) {

                                /* out of memory when unexpected */
                                if ( validSizeArray[j] == TRUE ) {
                                    cRetVal = 0;
                                    TST_verbose("Error: out of memory.\n");
                                }
                            }
                            /* verify that if error, pBuf = NULL */
                            else if (!DSP_SUCCEEDED(status) &&
                                (*arg4[l]) != NULL ) {
                                TST_verbose("Warning:[%d][%d][%d][%d]: ",
                                    i, j, k, l);
                                TST_verbose("pBuf should be NULL\n");

                                if (bUseLog) {
                                    TSTLOG_appendLog(pHandle, "FAIL\n");
                                    TSTLOG_appendLog(pHandle,
                                        "DSPNode_AllocMsgBuf");
                                    TSTLOG_appendLog(pHandle,
                                        "[%d][%d][%d][%d]", i, j, k, l);
                                    TSTLOG_appendLog(pHandle,
                                        "Should NULL pBuf when failing.\n");
                                }
                            }
                        }
                        
                        /* check if match expected error codes */
                        if (!TST_CheckVal(status,
                            checkArray[i][j][k][l].dwValue,
                            checkArray[i][j][k][l].fEqualFlag) ) {
                            cRetVal = 0;
                            TST_verbose("Error: Values don't match\n");
                            
                            if (bUseLog) {
                                TSTLOG_appendLog(pHandle, "FAIL\n");
                                TSTLOG_appendLog(pHandle,
                                    "DSPNode_AllocMsgBuf");
                                TSTLOG_appendLog(pHandle,
                                    "[%d][%d][%d][%d]", i, j, k, l);
                                TSTLOG_appendLog(pHandle,
                                    "failed.\n");
                            }
                        }
                        
                        if (DSP_SUCCEEDED(status)) {
                            DSPNode_FreeMsgBuf(arg1[i], *arg4[l], arg3[k]);
                        }
                    }
                }
            }
        }
        status = DSPNode_Terminate(arg1[0], &exitStatus);
        break;
    }

    status = DSPNode_Delete(arg1[0]);
    status = DSPProcessor_Detach(hProcessor);


    OsSpecificMemFree(pAttrIn);
	OsSpecificMemFree(arg3[0]);
	OsSpecificMemFree(arg3[1]);
	OsSpecificMemFree(arg3[2]);
	OsSpecificMemFree(arg3[3]);
	OsSpecificMemFree(arg4[0]);
    OsSpecificMemFree(nodeAttrin);
    OsSpecificMemFree(cbData);

    if (cRetVal) {
         /* Report in LOG module */
         if (bUseLog) {
             TSTLOG_appendLog(pHandle, "PASS\n");
         }
         TST_verbose("PASS\n");
    }
     else {
         /* Report in LOG module */
         if (bUseLog) {
             TSTLOG_appendLog(pHandle, "FAIL\n");
         }
         TST_verbose("FAIL\n");
     }

    /* Close Log */
    if (bUseLog) {
         TSTLOG_closeLog(pHandle);
    }

    return 0;
}

void ivaoutStatus(UINT status)
{
    switch(status) {
    case 0x8000:
        TST_verbose("DSP_SOK       ");
        break;
    case 0x80008012:
        TST_verbose("DSP_EPOINTER  ");
        break;
    case 0x80008034:
        TST_verbose("DSP_EALIGNMENT");
        break;
    case 0x80008035:
        TST_verbose("DSP_EBADSEGID ");
        break;
    case 0x8000800a:
        TST_verbose("DSP_EHANDLE   ");
        break;
    case 0x80008014:
        TST_verbose("DSP_SIZE      ");
        break;
    default:
        TST_verbose("%x", status);
    }
}
