/*
 *  Copyright 2004 by Texas Instruments Incorporated.
 *  All rights reserved. Property of Texas Instruments Incorporated.
 *  Restricted rights to use, duplicate or disclose this code are
 *  granted through contract.
 *
 *  @(#) DSP/BIOS Bridge 
 */
/*  ============================================================================
 *   File    uuidutil.c
 *
 *   Path    $(TESTSUITEROOT)\Testutil\tstlog\uuidutil.c
 * 
 *   Desc:
 *      This file contains the implementation of UUID helper functions.
 *
 *   ---------------------------------------------------------------------------
 *   Rev     1.0.0
 * 
 *   This computer program is copyright to Texas Instruments Incorporated.
 *   The program may not be used without the written permission
 *   of Texas Instruments Incorporated. or against the terms and conditions
 *   stipulated in the agreement under which this program has been
 *   supplied.
 *
 *   (c) Texas Instruments Incorporated 2003
 *
 *   ============================================================================
 *   Revision History
 *
 *   JUL xx, 2002    REF=ORG             Swamy MC
 *      Original Version.(Taken from WinCE Test Suite)
 *   ============================================================================
 */


/*  ----------------------------------- Host OS  */
#include <stdio.h>

/*  ----------------------------------- DSP/BIOS Bridge */
#include <std.h>
#include <dbdefs.h>

/*  ----------------------------------- This */
#include <uuidutil.h>




/*
 *  ======== UUID_UuidToString ========
 *  Purpose:
 *      Converts a struct DSP_UUID to a string.
 */
VOID UUID_UuidToString(IN struct DSP_UUID *pUuid, OUT CHAR *pszUuid,INT size)
{
    INT     i;
    
    i = sprintf(pszUuid, "%.8lX_%.4X_%.4X_%.2X%.2X_%.2X%.2X%.2X%.2X%.2X%.2X",
        pUuid->ulData1, 
        pUuid->usData2, 
        pUuid->usData3, 
        pUuid->ucData4, pUuid->ucData5,
        pUuid->ucData6[0], pUuid->ucData6[1], pUuid->ucData6[2], 
        pUuid->ucData6[3], pUuid->ucData6[4], pUuid->ucData6[5]);

}

/*
 *  ======== htoi ========
 *  purpose:
 *      Converts a hex value to a decimal integer.
 */

int htoi(char c)
{
	switch(c)
	{
		case '0': return 0;
		case '1': return 1;
		case '2': return 2;
		case '3': return 3;
		case '4': return 4;
		case '5': return 5;
		case '6': return 6;
		case '7': return 7;
		case '8': return 8;
		case '9': return 9;
		case 'A': return 10;
		case 'B': return 11;
		case 'C': return 12;
		case 'D': return 13;
		case 'E': return 14;
		case 'F': return 15;
		case 'a': return 10;
		case 'b': return 11;
		case 'c': return 12;
		case 'd': return 13;
		case 'e': return 14;
		case 'f': return 15;
	}
	return 0;
}

/*
 *  ======== UUID_UuidFromString ========
 *  purpose:
 *      Converts a string to a struct DSP_UUID.
 */
VOID UUID_UuidFromString(IN CHAR *pszUuid,  OUT struct DSP_UUID *pUuid)
{
	char c;
	int i,j;
	long result;
	char *temp = pszUuid;

	result = 0;
	for(i=0;i<8;i++)
	{
		// Get first character in string
		c = *temp;
		
		// Increase the results by new value
		result *= 16;
		result += htoi(c);

		// Go to next character in string
		temp++;
	}
	pUuid->ulData1 = result;

	// Step over underscore
	temp++;

	result = 0;
	for(i=0;i<4;i++)
	{
		// Get first character in string
		c = *temp;
		
		// Increase the results by new value
		result *= 16;
		result += htoi(c);

		// Go to next character in string
		temp++;
	}
	pUuid->usData2 = result;

	// Step over underscore
	temp++;

	result = 0;
	for(i=0;i<4;i++)
	{
		// Get first character in string
		c = *temp;
		
		// Increase the results by new value
		result *= 16;
		result += htoi(c);

		// Go to next character in string
		temp++;
	}
	pUuid->usData3 = result;


	// Step over underscore
	temp++;

	result = 0;
	for(i=0;i<2;i++)
	{
		// Get first character in string
		c = *temp;
		
		// Increase the results by new value
		result *= 16;
		result += htoi(c);

		// Go to next character in string
		temp++;
	}
	pUuid->ucData4 = result;

	result = 0;
	for(i=0;i<2;i++)
	{
		// Get first character in string
		c = *temp;
		
		// Increase the results by new value
		result *= 16;
		result += htoi(c);

		// Go to next character in string
		temp++;
	}
	pUuid->ucData5 = result;

	// Step over underscore
	temp++;

	for(j=0;j<6;j++)
	{
		result = 0;
		for(i=0;i<2;i++)
		{
			// Get first character in string
			c = *temp;
			
			// Increase the results by new value
			result *= 16;
			result += htoi(c);

			// Go to next character in string
			temp++;
		}
		pUuid->ucData6[j] = result;
	}
}

